;-------------------------------------------------------------
;+
; NAME:
;       DEFINED
; PURPOSE:
;       Returns 1, if expression is defined, 0 if it is a undefined variable.
; CATEGORY:
;       Informational Routines
; CALLING SEQUENCE:
;       result=defined, EXPR [, TAG=TAG] [,STRUCTURE=STRUCTURE]
; INPUTS:
;       EXPR=Expression to be examined
;       TAG=Tag of the strucure 'expr'
; KEYWORD PARAMETERS:
;       Keywords:
;       TAG=Tag of the strucure 'expr'. If this keyword is set,
;         not expr is examined being defined, but 'expr.tag'. This is only
;         valid, if the keyword 'structure' is set.
;       STRUCTURE=If this keyword is set, 1 is returned, if 'expr' is
;         a structure or expr.tag is valid, 0 if not.
; OUTPUTS:
;       Returns 1 if expr or expr.tag is defined, 0 if not.
; MODIFICATION HISTORY:
;       Matthias Liebetanz, TU-Darmstadt, Germany, 31 Mrz, 1997
;       No warranty that this program is free of error
;
;-
;-------------------------------------------------------------
FUNCTION DEFINED, expr, TAG=tag, STRUCTURE=STRUCTURE

      expr_info=SIZE(expr)
      info_length=N_ELEMENTS(expr_info)
      expr_type=expr_info(info_length-2)                                    ; Get information about the type of state

      IF KEYWORD_SET(STRUCTURE) THEN BEGIN
       IF expr_type EQ 8 THEN BEGIN                                         ; If it is a structure
        IF KEYWORD_SET(TAG) THEN BEGIN
          tags_expr=TAG_NAMES(expr)
          index = WHERE(tags_expr EQ UPCASE(tag), count)
          IF count NE 0 THEN RETURN, 1 $                                    ; TAG is defined
           ELSE RETURN, 0                                                   ; TAG is not defined
        ENDIF ELSE RETURN, 1                                                ; IS a structure
       ENDIF ELSE RETURN, 0                                                 ; Is NOT a structure
      ENDIF ELSE BEGIN                                                      ; Keyword 'STRUCTURE' is not set
       IF N_ELEMENTS(expr) NE 0 THEN RETURN, 1  $                           ; expr is defined
        ELSE RETURN,0                                                       ; expr is NOT defined
      ENDELSE
END

