;-------------------------------------------------------------
;+
; NAME:
;       IPOLBOX
; PURPOSE:
;       Interpolates linear from a box given in device-coordinates to one in data-coordinates
; CATEGORY:
;       Math
; CALLING SEQUENCE:
;       newdatabox=IPOLBOX(oldbox,databox,newbox)
; INPUTS:
;       oldbox=borders of original box in device-coordinates oldbox=[x1min,y1min,x2max,y2max]
;       newbox=borders of target box in device-coordinates oldbox=[x1min,y1min,x2max,y2max]
;       databox=borders of original box in data-coordinates oldbox=[x1min,y1min,x2max,y2max]
; KEYWORD PARAMETERS:
;       Keywords:
;       TITLE=If this keyword is set, the string becomes the title of the widged-window.
;       MODAL=If this keyword is set, CW_SHOWTEXT is blocking all further processing,
;       until it is destroyed.
; OUTPUTS:
; COMMON BLOCKS:
; NOTES:
; MODIFICATION HISTORY:
;       Matthias Liebetanz, 31 Mrz, 1997
;-
;-------------------------------------------------------------
FUNCTION IPOLBOX,oldbox,databox,newbox



oldbox=oldbox*1.D
databox=databox*1.D
newbox=newbox*1.D

xoldsize=oldbox(2)-oldbox(0)
yoldsize=oldbox(3)-oldbox(1)
xdatasize=databox(1)-databox(0)
ydatasize=databox(3)-databox(2)

xfaktor=xdatasize/xoldsize
yfaktor=ydatasize/yoldsize
newdatabox=DBLARR(4)
newdatabox(0)=databox(0)+(newbox(0)-oldbox(0))*xfaktor
newdatabox(1)=databox(1)+(newbox(1)-oldbox(2))*xfaktor
newdatabox(2)=databox(2)-(oldbox(1)-newbox(2))*yfaktor
newdatabox(3)=databox(3)-(oldbox(3)-newbox(3))*yfaktor

RETURN,newdatabox

END




