;------------------------------------------------------------- 
;+ 
; NAME: 
;       LOCERR 
; PURPOSE: 
;       This routine creates a widget interface for editing 
;       localisation errors of earthquakes 
; MAIN PROGRAM: 
;       SLAP - program for testing SEISMOLAP  
;       SEISMOLAP was developed by Jochen Zschau, 
;       Geoforschungszentrum Potsdam, Germany 
; MODIFICATION HISTORY: 
;       Matthias Liebetanz, TH-Darmstadt, Germany, 17. Apr 1998 . 
; 
;       No warranty that this program is free of error 
;- 
;------------------------------------------------------------- 
FUNCTION LOCERR, GROUP=group, $ 
                 ALTERED=altered,$ 
                 INITIALIZE=INITIALIZE,$ 
                 SIGXY_JS=xy,$ 
                 SIGZ=z, $ 
                 SIGT=t, $ 
                 QUAKE_QUALITY=quality 
 
IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0 
 
min_factor = MIN ( quality, MAX = max_factor ) 
n_factor = max_factor - min_factor + 1 
min_factor = MIN ( quality, MAX = max_factor ) 
 
IF KEYWORD_SET(INITIALIZE) THEN BEGIN 
   xy=DBLARR(n_factor) 
   z=xy 
   t=xy 
   FOR i=0,n_factor-1 DO BEGIN 
       xy(i) = 1000.D 
       z(i) = 1000.D 
       t(i) = 0.001D 
   ENDFOR 
   error = CREATE_STRUCT ( 'xy', xy, $ 
                           'z', z, $ 
                           't', t ) 
   RETURN, error 
ENDIF 
 
altered = 0 
xy_old = xy 
z_old = z 
t_old = t 
 
junk   = { CW_PDMENU_S, flags:0, name:'' } 
 
  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $ 
      COLUMN=1, $ 
      MAP=1, $ 
      TITLE='Localisation error', $ 
      UVALUE='MAIN13') 
 
  BASE34 = WIDGET_BASE(MAIN13, $ 
      ROW=1, $ 
      MAP=1, $ 
      UVALUE='BASE34') 
 
  BASE42 = WIDGET_BASE(BASE34, $ 
      COLUMN=1, $ 
      FRAME=1, $ 
      MAP=1, $ 
      UVALUE='BASE42') 
 
  LABEL40 = WIDGET_LABEL( BASE42, $ 
      UVALUE='LABEL40', $ 
      VALUE='Quality factor') 
 
  Btns4079 = STRARR(n_factor) 
 
  FOR i=0,n_factor -1 DO BEGIN 
      Btns4079(i) = STRTRIM(STRING(i),1) 
  ENDFOR 
 
  BUTTON_QUALITY = CW_BGROUP( BASE42, Btns4079, $ 
      COLUMN=1, $ 
      /NO_RELEASE, $ 
      EXCLUSIVE=1, $ 
      UVALUE='BUTTON_QUALITY') 
 
 
  BASE39 = WIDGET_BASE(BASE34, $ 
      COLUMN=1, $ 
      FRAME=1, $ 
      MAP=1, $ 
      UVALUE='BASE39') 
 
  LABEL35 = WIDGET_LABEL( BASE39, $ 
      UVALUE='LABEL35', $ 
      VALUE='Error values') 
 
  FieldVal4083 = [ $ 
    '' ] 
  FIELD_XY_ERROR = CW_FIELD( BASE39,VALUE=FieldVal4083, $ 
      ROW=1, $ 
      FLOAT=1, $ 
      ALL_EVENTS=1, $ 
      TITLE='x - y - error [m]', $ 
      UVALUE='FIELD_XY_ERROR') 
 
  FieldVal4085 = [ $ 
    '' ] 
  FIELD_Z_ERROR = CW_FIELD( BASE39,VALUE=FieldVal4085, $ 
      ROW=1, $ 
      FLOAT=1, $ 
      ALL_EVENTS=1, $ 
      TITLE='z - error [m].....', $ 
      UVALUE='FIELD_Z_ERROR') 
 
  FieldVal4087 = [ $ 
    '' ] 
  FIELD_T_ERROR = CW_FIELD( BASE39,VALUE=FieldVal4087, $ 
      ROW=1, $ 
      FLOAT=1, $ 
      ALL_EVENTS=1, $ 
      TITLE='time - error [s].', $ 
      UVALUE='FIELD_T_ERROR') 
 
  BUTTON45 = WIDGET_BUTTON( MAIN13, $ 
      UVALUE='BUTTON_CANCEL', $ 
      VALUE='Cancel') 
 
  BUTTON46 = WIDGET_BUTTON( MAIN13, $ 
      UVALUE='BUTTON_OK', $ 
      VALUE='Ok') 
 
  WIDGET_CONTROL, MAIN13, /REALIZE 
 
 
 
Ev='' 
 
WHILE (Ev NE 'BUTTON_OK') AND (Ev NE 'BUTTON_CANCEL') DO BEGIN 
 
Event=WIDGET_EVENT(MAIN13) 
 
WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev 
 
CASE Ev OF 
 
  'BUTTON_QUALITY': BEGIN 
      WIDGET_CONTROL, FIELD_XY_ERROR, SET_VALUE = xy(event.value) 
      WIDGET_CONTROL, FIELD_Z_ERROR, SET_VALUE = z(event.value) 
      WIDGET_CONTROL, FIELD_T_ERROR, SET_VALUE = t(event.value) 
      current_index = event.value 
      END 
  'FIELD_XY_ERROR': BEGIN 
      xy(current_index) = event.value 
      altered = 1 
      END 
  'FIELD_Z_ERROR': BEGIN 
      z(current_index) = event.value 
      altered = 1 
      END 
  'FIELD_T_ERROR': BEGIN 
      t(current_index) = event.value 
      altered = 1 
      END 
  'BUTTON_CANCEL': BEGIN 
      END 
  'BUTTON_OK': BEGIN 
      END 
 
ENDCASE 
 
ENDWHILE 
 
IF Ev EQ 'BUTTON_OK' THEN BEGIN 
 
   create_structure: 
 
   IF altered THEN BEGIN 
 
   error = CREATE_STRUCT ( 'xy', xy, $ 
                           'z', z, $ 
                           't', t ) 
 
   ENDIF ELSE  error=0 
 
ENDIF ELSE error=0 
 
IF DEFINED(MAIN13) THEN WIDGET_CONTROL,MAIN13,/DESTROY 
 
RETURN,TEMPORARY(error) 
 
END 
