;-------------------------------------------------------------
;+
; NAME:
;       PARAM
; PURPOSE:
;       This routine creates a widget based interface for setting SEISMOLAP - Parameters
; CATEGORY:
;       Widgets
; DEPENDENCIES:
;       TIMEWIN
;       SPATIALWIN     
;       MAINSHCK
;       GRIDINIT
; MODIFICATION HISTORY:
;       Matthias Liebetanz, Darmstadt University of Technology, Germany, 19 May 1998.
;
;-
;-------------------------------------------------------------
FUNCTION PARAM,GROUP=Group,$
            ALTERED=altered,$
            CHANGES_DONE=changes_done,$
            INITIALIZE=initialize,$
            AREA_BORDER=area_border,$
            GRID_LON=grid_lon,$
            GRID_LAT=grid_lat, $
            GRID_BORDER=grid_border,$
            GRID_SPACING=grid_spacing,$
            Z_SPACING=z_spacing,$
            GRID_LENGTH=grid_length,$
            GRID_Z=grid_z,$
            GRID_DATE=grid_date,$
            GRID_TIMEWIN=grid_timewin,$
            GRID_SHIFT=grid_shift,$
            QUAKE_LENGTH=quake_length,$
            QUAKE_JS=quake_js,$
            QUAKE_LON=quake_lon,$
            QUAKE_LAT=quake_lat,$
            QUAKE_Z=quake_z,$
            QUAKE_STRENGTH=quake_strength,$
            SWITCH_POINT_MODE=switch_point_mode,$
            SWITCH_DEPTH_RANGE=switch_depth_range,$
            SWITCH_QUAKE_NE_GRID=switch_quake_ne_grid,$
            SWITCH_KEYBOARD=switch_keyboard,$
            SWITCH_MOUSE=switch_mouse,$
            SWITCH_NORMALIZE=switch_normalize,$
            SWITCH_QUIESCENCE=switch_quiescence,$
            SWITCH_ERROR=switch_error

IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

IF KEYWORD_SET(INITIALIZE) THEN BEGIN

   TIMEWIN, GROUP=Group,$
            /INIT,$
            GRID_DATE=grid_date,$
            QUAKE_JS=quake_js,$
            GRID_TIMEWIN=grid_timewin,$
            GRID_SHIFT=grid_shift

   SPATIALWIN, GROUP=Group,$
            /INIT,$
            AREA_BORDER=area_border,$
            GRID_BORDER=grid_border,$
            GRID_SPACING=grid_spacing,$
            Z_SPACING=z_spacing,$
            GRID_LENGTH=grid_length,$
            GRID_LON=grid_lon,$
            GRID_LAT=grid_lat,$
            GRID_Z=grid_z,$
            QUAKE_LON=quake_lon,$
            QUAKE_LAT=quake_lat,$
            QUAKE_Z=quake_z,$
            QUAKE_LENGTH=quake_length,$
            SWITCH_POINT_MODE=switch_point_mode,$
            SWITCH_DEPTH_RANGE=switch_depth_range,$
            SWITCH_QUAKE_NE_GRID=switch_quake_ne_grid,$
            SWITCH_KEYBOARD=switch_keyboard,$
            SWITCH_MOUSE=switch_mouse,$
            SWITCH_NORMALIZE=switch_normalize,$
            SWITCH_QUIESCENCE=switch_quiescence,$
            SWITCH_ERROR=switch_error

ENDIF

altered=0
altered_time=0
altered_place=0
altered_mainshocks=0

minjs=MIN(quake_js,MAX=maxjs)
mindate=dt_tm_fromjs(minjs,FORMAT='0d$ n$ Y$  h$:m$:s$')  ;dt_tm_fromjs, jhu-library
maxdate=dt_tm_fromjs(maxjs,FORMAT='0d$ n$ Y$  h$:m$:s$')

IF KEYWORD_SET(INITIALIZE) THEN GOTO,create_structure

junk   = { CW_PDMENU_S, flags:0, name:'' }

  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, $
      TITLE='Parameters', $
      UVALUE='MAIN13')

  BASE107 = WIDGET_BASE(MAIN13, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE107')

  LABEL105 = WIDGET_LABEL( BASE107, $
      UVALUE='LABEL105', $
      VALUE='Events:')

  LABEL106 = WIDGET_LABEL( BASE107, $
      VALUE=STRING(N_ELEMENTS(quake_lon)), $
      UVALUE='LABEL106')


  BASE104 = WIDGET_BASE(MAIN13, $
      COLUMN=1, $
      FRAME=1, $
      MAP=1, $
      UVALUE='BASE104')

  LABEL150 = WIDGET_LABEL( BASE104, $
      UVALUE='LABEL150', $
      VALUE='S P A T I A L   B O R D E R')

  BASE91 = WIDGET_BASE(BASE104, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE91')

  LABEL92 = WIDGET_LABEL( BASE91, $
      UVALUE='LABEL92', $
      VALUE='min latitude:')

  LABEL93 = WIDGET_LABEL( BASE91, $
      VALUE=STRING(grid_border(2)), $
      UVALUE='LABEL93')


  BASE88 = WIDGET_BASE(BASE104, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE88')

  LABEL94 = WIDGET_LABEL( BASE88, $
      UVALUE='LABEL94', $
      VALUE='max latitude:')

  LABEL95 = WIDGET_LABEL( BASE88, $
      VALUE=STRING(grid_border(3)),$
      UVALUE='LABEL95')


  BASE85 = WIDGET_BASE(BASE104, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE85')

  LABEL96 = WIDGET_LABEL( BASE85, $
      UVALUE='LABEL96', $
      VALUE='min longitude:')

  LABEL97 = WIDGET_LABEL( BASE85, $
      VALUE=STRING(grid_border(0)), $
      UVALUE='LABEL97')


  BASE82 = WIDGET_BASE(BASE104, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE82')

  LABEL98 = WIDGET_LABEL( BASE82, $
      UVALUE='LABEL98', $
      VALUE='max longitude:')

  LABEL99 = WIDGET_LABEL( BASE82, $
      VALUE=STRING(grid_border(1)), $
      UVALUE='LABEL99')


  BASE79 = WIDGET_BASE(BASE104, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE79')

  LABEL100 = WIDGET_LABEL( BASE79, $
      UVALUE='LABEL100', $
      VALUE='min depth [km]:')

  LABEL101 = WIDGET_LABEL( BASE79, $
      VALUE=STRING(grid_border(4)), $
      UVALUE='LABEL101')


  BASE76 = WIDGET_BASE(BASE104, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE76')

  LABEL102 = WIDGET_LABEL( BASE76, $
      UVALUE='LABEL102', $
      VALUE='max depth [km]:')

  LABEL103 = WIDGET_LABEL( BASE76, $
      VALUE=STRING(grid_border(5)), $
      UVALUE='LABEL103')



  BASE133 = WIDGET_BASE(MAIN13, $
      COLUMN=1, $
      FRAME=1, $
      MAP=1, $
      UVALUE='BASE133')

  LABEL154 = WIDGET_LABEL( BASE133, $
      UVALUE='LABEL154', $
      VALUE='T I M E   B O R D E R')

  BASE118 = WIDGET_BASE(BASE133, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE118')

  LABEL129 = WIDGET_LABEL( BASE118, $
      UVALUE='LABEL129', $
      VALUE='first date:')

  LABEL130 = WIDGET_LABEL( BASE118, $
      VALUE=mindate, $
      UVALUE='LABEL130')


  BASE115 = WIDGET_BASE(BASE133, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE115')

  LABEL131 = WIDGET_LABEL( BASE115, $
      UVALUE='LABEL131', $
      VALUE='last date:')

  LABEL132 = WIDGET_LABEL( BASE115, $
      VALUE=maxdate,$
      UVALUE='LABEL132')



  LABEL135 = WIDGET_LABEL( MAIN13, $
      UVALUE='LABEL135', $
      VALUE='')

  BUTTON47 = WIDGET_BUTTON( MAIN13, $
      UVALUE='BUTTON_SPATIAL_WINDOW', $
      VALUE='Set spatial window')

  LABEL50 = WIDGET_LABEL( MAIN13, $
      UVALUE='LABEL50', $
      VALUE='')

  BUTTON109 = WIDGET_BUTTON( MAIN13, $
      UVALUE='BUTTON_TIME_WINDOW', $
      VALUE='Set time window')

  LABEL111 = WIDGET_LABEL( MAIN13, $
      UVALUE='LABEL111', $
      VALUE='')

  BUTTON120 = WIDGET_BUTTON( MAIN13, $
      UVALUE='BUTTON_MAIN_EVENTS', $
      VALUE='Main events')

  LABEL111 = WIDGET_LABEL( MAIN13, $
      UVALUE='LABEL111', $
      VALUE='')

  BUTTON37 = WIDGET_BUTTON( MAIN13, $
      UVALUE='BUTTON_CANCEL', $
      VALUE='Cancel')

  BUTTON40 = WIDGET_BUTTON( MAIN13, $
      UVALUE='BUTTON_OK', $
      VALUE='Ok')

  WIDGET_CONTROL, MAIN13, /REALIZE


Ev=''

WHILE (Ev NE 'BUTTON_OK') AND (Ev NE 'BUTTON_CANCEL') DO BEGIN

Event=WIDGET_EVENT(MAIN13)

WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

CASE Ev OF

  'BUTTON_TIME_WINDOW': BEGIN
      TIMEWIN, GROUP=Group,$
            ALTERED=altered_time,$
            GRID_DATE=grid_date,$
            QUAKE_JS=quake_js,$
            GRID_TIMEWIN=grid_timewin,$
            GRID_SHIFT=grid_shift
      END
  'BUTTON_SPATIAL_WINDOW': BEGIN
      SPATIALWIN, GROUP=Group,$
            ALTERED=altered_place,$
            AREA_BORDER=area_border,$
            GRID_BORDER=grid_border,$
            GRID_SPACING=grid_spacing,$
            Z_SPACING=z_spacing,$
            GRID_LENGTH=grid_length,$
            GRID_LON=grid_lon,$
            GRID_LAT=grid_lat,$
            GRID_Z=grid_z,$
            QUAKE_LENGTH=quake_length,$
            QUAKE_LON=quake_lon,$
            QUAKE_LAT=quake_lat,$
            QUAKE_Z=quake_z,$
            SWITCH_POINT_MODE=switch_point_mode,$
            SWITCH_DEPTH_RANGE=switch_depth_range,$
            SWITCH_QUAKE_NE_GRID=switch_quake_ne_grid,$
            SWITCH_KEYBOARD=switch_keyboard,$
            SWITCH_MOUSE=switch_mouse,$
            SWITCH_NORMALIZE=switch_normalize,$
            SWITCH_QUIESCENCE=switch_quiescence,$
            SWITCH_ERROR=switch_error


      END
  'BUTTON_MAIN_EVENTS': BEGIN
      MAINSHCK, ALTERED=altered_mainshocks, $
                QUAKE_LON=quake_lon,$
                QUAKE_LAT=quake_lat,$
                QUAKE_Z=quake_z,$
                QUAKE_STRENGTH=quake_strength,$
                QUAKE_JS=quake_js,$
                INFO_SHOCK=info_shock,$
                SELECTED_SCHOCK=selected_shock,$
                FOCUS=focus

      IF focus THEN BEGIN
         area_border(0:1)=info_shock.lon(selected_shock)
         area_border(2:3)=info_shock.lat(selected_shock)
         area_border(4:5)=info_shock.z(selected_shock)
         switch_point_mode=1
         switch_depth_range=0
         GRIDINIT,grid_spacing,z_spacing,area_border,grid_lon,grid_lat,grid_z
         altered_place=1
      ENDIF


      END
  'BUTTON_CANCEL': BEGIN
      END
  'BUTTON_OK': BEGIN
      END

ENDCASE

ENDWHILE

IF Ev EQ 'BUTTON_OK' THEN BEGIN

create_structure:

   IF altered_time OR altered_place OR altered_mainshocks THEN altered=1

   IF NOT DEFINED(changes_done) THEN changes_done=0

   IF (changes_done EQ 1) AND (altered NE 0) THEN changes_done=0

   area=CREATE_STRUCT ( $
             'border',area_border $
             )

   grid=CREATE_STRUCT ( $
             'border',grid_border,$
             'lon',grid_lon,$
             'lat',grid_lat,$
             'z',grid_z,$
             'spacing',grid_spacing,$
             'z_spacing',z_spacing,$
             'length',grid_length,$
             'timewin',grid_timewin,$
             'shift',grid_shift,$
             'date',grid_date $
             )

   quake=CREATE_STRUCT ( $
             'length',quake_length $
             )

   config=CREATE_STRUCT( $
             'switch_point_mode',switch_point_mode, $
             'switch_depth_range',switch_depth_range, $
             'switch_keyboard',switch_keyboard,$
             'switch_mouse',switch_mouse, $
             'switch_quake_ne_grid',switch_quake_ne_grid, $
             'switch_normalize',switch_normalize,$
             'switch_quiescence',switch_quiescence,$
             'switch_error',switch_error $
             )

   param=CREATE_STRUCT ( $
             'altered',altered,$
             'changes_done',changes_done, $
             'area',TEMPORARY(area),$
             'grid',TEMPORARY(grid),$
             'quake',TEMPORARY(quake),$
             'config',TEMPORARY(config) $
             )

ENDIF ELSE  param=0

IF DEFINED(MAIN13) THEN WIDGET_CONTROL,MAIN13,/DESTROY
RETURN,TEMPORARY(param)
END

