;------------------------------------------------------------- 
;+ 
; NAME: 
;       SEC2HMS 
; PURPOSE: 
;       Convert seconds after midnight to a time string. 
; CATEGORY: 
; CALLING SEQUENCE: 
;       tstr = strsec(sec, [d]) 
; INPUTS: 
;       sec = seconds after midnight.             in 
;         Scalar or array. 
;       d = optional denominator for a fraction.  in 
; KEYWORD PARAMETERS: 
; OUTPUTS: 
; COMMON BLOCKS: 
; NOTES: 
; MODIFICATION HISTORY: 
;       Modified Function ('strsec') original written by: 
;       R. Sterner, 8 Jan, 1985. 
;       Johns Hopkins University Applied Physics Laboratory. 
; 
; Copyright (C) 1985, Johns Hopkins University/Applied Physics Laboratory 
; This software may be used, copied, or redistributed as long as it is not 
; sold and this copyright notice is reproduced on each copy made.  This 
; routine is provided as is without any express or implied warranties 
; whatsoever.  Other limitations apply as described in the file disclaimer.txt. 
;- 
;------------------------------------------------------------- 
  
        pro sec2hms,sec0, h,m,s 
  
  
        seca = sec0 
        nn = n_elements(seca)           ; Number of elements in input array. 
  
        ;------------------------------------------------------------------- 
        for ii = 0, nn-1 do begin       ; Loop through all input elements. 
        sec = seca(ii)                  ; Pull out the ii'th element. 
        t = double(sec)                 ; Convert to double. 
        dy = long(t/86400)              ; # days. 
        t = t - 86400*dy                ; Time without days. 
        h = long(t/3600)                ; # hours. 
        t = t - 3600*h                  ; Time without hours. 
            h = h + 24*dy 
        dy = 0 
        m = long(t/60)                  ; # minutes. 
        t = t - 60*m                    ; Time without minutes. 
        s = long(t)                     ; Seconds. 
        f = t - s                       ; Time without seconds (=fraction). 
  
  
        endfor 
        ;------------------------------------------------------------------- 
  
  
  
        end 
