;-------------------------------------------------------------   
;+   
; NAME:   
;       SLAP   
; PURPOSE:   
;       Program for testing SEISMOLAP.   
;       SEISMOLAP was developed by Jochen Zschau,   
;       Geoforschungszentrum Potsdam, Germany   
; CATEGORY:   
;       Earthquake prediction   
; CALLING SEQUENCE:   
;       SLAP   
; MODIFICATION HISTORY:   
;       Matthias Liebetanz, TU-Darmstadt, Germany, 17 Apr, 1998.   
;       No warranty that this program is free of error   
;   
;-   
;-------------------------------------------------------------   
PRO PDMENU_FILE_Event, Event   
   
   
  CASE Event.Value OF   
   
   
  'File.Close': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )          ; Get child-ID   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY      ; Get state as user-value of child   
   
    IF DEFINED(state,/STRUCTURE,TAG='quake') THEN BEGIN   ; If state.quake is not defined   
       lastpath=state.lastpath   
       state=''                                           ; Reset state   
       state=CREATE_STRUCT('lastpath',lastpath)   
    ENDIF ELSE $   
       tmp=WIDGET_MESSAGE('No File open.',/ERROR)         ; No file open   
   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
   
    END   
  'File.Open': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )          ; Get child-ID   
   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY      ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='lastpath') THEN BEGIN   
       state=CREATE_STRUCT('lastpath','')   
       lastpath=''   
    ENDIF ELSE lastpath=state.lastpath   
   
    file = PICKFILE(TITLE='Open File',$                   ; Get name of data-file   
                     FILTER='*.sav',$   
                      PATH=lastpath,$   
                       GET_PATH=path)   
    IF file EQ '' THEN GOTO, end_open               ; 'Cancel' was pressed   
begin_open:   
    WIDGET_CONTROL, /HOURGLASS   
    extension=FEXT(file)   
    IF (extension EQ '.sav') OR (extension EQ '.SAV') THEN BEGIN                     ; Check extension. If it is   
      RESTORE, file                                       ; a 'sav'-file, then restore it.   
      IF NOT DEFINED(state,/STRUCTURE,TAG='lastpath') THEN $   
         state=CREATE_STRUCT(state,'lastpath',path) $   
      ELSE $   
         state.lastpath=path                              ; Store current path   
   
      GOTO, end_open   
    ENDIF   
    lastpath=path                                         ; Store current path   
    state.lastpath=path                                   ; Store directly in state, if it is not a valid database   
    headerline=''                                             ; Else check the haeder for   
    OPENR,unit,file,/GET_LUN                              ; data-format of the file   
    READF,unit,headerline   
    IF (headerline NE '#Seismolap testprogram') OR (EOF(unit)) THEN BEGIN   
       tmp=WIDGET_MESSAGE('This is not a valid database.',/ERROR)   
       FREE_LUN,unit   
       GOTO,end_open                                ; The first two lines mark the   
    ENDIF                                                 ; file as earthquake-related file   
    READF,unit,headerline   
    IF (headerline NE '#micro-earthquakes-data-file') OR (EOF(unit)) THEN BEGIN   
       tmp=WIDGET_MESSAGE('This is not a valid database.',/ERROR)   
       FREE_LUN,unit   
       GOTO,end_open   
    ENDIF   
    headerline3=''   
    READF,unit,headerline3                                 ; In the third line the format of data is described   
    READF,unit,headerline   
    result=FLINES(UNI=unit,/F_CLOSE)                       ; Count the lines and close the file   
    IF result(2) NE -1 THEN lines=result(1) $              ; If no error has occured on counting   
    ELSE GOTO,end_open   
    filepointer=FLINES(F_NAME=file,STR='#',RPEAT=3)          ; Reopen the file and set the pointer on first data-line   
    WIDGET_CONTROL, /HOURGLASS   
    CASE headerline3 OF   
         '#format:Ii,DD.MM.JJ,hh:mm:ss.ss,Flat,Flon,Fdepth,Fstrength,Iquality':$   
         BEGIN   
                str=STRARR(lines)   
                READF,unit,str                                         ; read file in str   
                SREPLACE,str,':',' ',FIRSTPOS=10,LASTPOS=25                           ; replace ':' with blanks in str   
                SREPLACE,str,'.',' ',LASTPOS=20   
                file=PICKFILE(/WRITE,PATH=lastpath,$   
                              FILE=FPRAEFIX(file)+'.tmp', $   
                              FILTER='*.tmp',$   
                              GET_PATH=lastpath)   
                IF file EQ '' THEN GOTO, end_open                ; 'Cancel' was pressed   
                OPENW,unit,file,/GET_LUN                               ; write the file to new format   
                header=['#Seismolap testprogram',$   
                        '#micro-earthquakes-data-file',$   
                        '#format:Ii,IDD,IMM,IJJ,Ihh,Imm,Fss.ss,Flat,Flon,Fdepth,Fstrength,Iquality',$   
                        '#generated:'+systime()]   
                PRINTF,unit,header,FORMAT='(A)'   
                PRINTF,unit,str   
                FREE_LUN,unit   
                GOTO,begin_open   
                END   
         '#format:Ii,IDD,IMM,IJJ,Ihh,Imm,Fss.ss,Flat,Flon,Fdepth,Fstrength,Iquality':$   
         BEGIN   
                _number =0L   
                _day = 0L  &  _month = 0L  &  _year = 0L                ; date of the earthquake   
                _hour = 0L &  _minute = 0L & _second = 0.D              ; time of the earthquake                           ; hour of the earthquake   
                _latitude  = 0.0D  &  _longitude = 0.0D                 ; geographic coordinates of the earthquake   
                _depth = 0.0                                            ; depth (in km) of the earthquake   
                _strength = 0.0                                         ; strength (Mercalli-Scala) of the earthquake   
                _quality =  0                                           ; quality of localisation ( 0(no), 1(<2km) - 4(>10km), 99(fiktive) )   
                number=LONARR(lines)   
                day=BYTARR(lines) & month=BYTARR(lines) & year=BYTARR(lines)   
                hour=BYTARR(lines) & minute=BYTARR(lines) & second=BYTARR(lines)   
                latitude=DBLARR(lines) & longitude=DBLARR(lines) & depth=DBLARR(lines)   
                strength=FLTARR(lines) & quality=INTARR(lines)   
                i=0L   
                WHILE NOT EOF(unit) DO BEGIN   
                      READF, unit, _number, $   
                                   _day,   $   
                                   _month,  $   
                                   _year, $   
                                   _hour,   $   
                                   _minute, $   
                                   _second, $   
                                   _latitude,  $   
                                   _longitude, $   
                                   _depth, $   
                                   _strength, $   
                                   _quality   
                      number(i)=_number   
                      day(i)=_day & month(i)=_month & year(i)=_year   
                      hour(i)=_hour & minute(i)=_minute & second(i)=_second   
                      latitude(i)=_latitude & longitude(i)=_longitude   
                      depth(i)=_depth & strength(i)=_strength   
                      quality(i)=_quality   
                      i=i+1   
                ENDWHILE   
                FREE_LUN,unit   
                info = WIDGET_MESSAGE ( STRING(i)+' Earthquakes readed.',/INFORMATION)   
                IF (year(0) LT 100) THEN year=year+1900   
                julsec=YMDS2JS(TEMPORARY(year), $                       ; julsec = "Julian Second". Routine taken from JHU-Library.   
                               TEMPORARY(month), $                      ; Has same purpose as Julian Days, easy handling time intervals.   
                               TEMPORARY(day), $                        ; The zero point is 0:00 1 Jan 2000, so js < 0 before then.   
                               TEMPORARY(hour*3600L+minute*60L+second)) ; Julian Seconds are double precision and have a precision   
                                                                        ; better than 1 millisecond over a span of +/- 1000 years.   
                END   
         ELSE : BEGIN   
                error=WIDGET_MESSAGE('This is not a valid database.',/ERROR)   
                GOTO,end_open   
                END   
    ENDCASE   
   
    quake = CREATE_STRUCT ('js', TEMPORARY(julsec(0:i-1)),$   
                           'lat', TEMPORARY(latitude),$   
                           'lon', TEMPORARY(longitude),$   
                           'z', TEMPORARY(depth(0:i-1)*1000.),$   ;Assume, depth is in [km]   
                           'strength', TEMPORARY(strength(0:i-1)),$   
                           'quality', TEMPORARY(quality(0:i-1)),$   
                           'n',i)                             ; number of earthqaukes (scalar)   
   
    state = CREATE_STRUCT ('quake',quake, $                ; Store data in state   
                           'file',file,$   
                           'lastpath',lastpath)   
   
   
   
end_open:                                            ; End-Label if an error occurs   
   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY       ; Set state as user-value of child   
   
    END   
  'File.Save as': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )           ; Get child-ID   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY       ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='quake') THEN BEGIN            ; If state is not a structure   
       tmp=WIDGET_MESSAGE('No File open.',/ERROR)        ; No file open   
   
       GOTO,exit_save_as   
    ENDIF   
   
    IF DEFINED(state,/STRUCTURE,TAG='lastpath') THEN lastpath=state.lastpath $   
    ELSE lastpath=''   
    answer = 'N'   
    WHILE answer EQ 'N' DO BEGIN   
          file = PICKFILE(/WRITE, $   
                    PATH=lastpath, $   
                    FILE=FPRAEFIX(state.file)+'.sav', $   
                    FILTER='*.sav',$   
                    GET_PATH=path)   
          IF file EQ '' THEN GOTO, exit_save_as $                ; 'Cancel' was pressed   
          ELSE BEGIN   
             answer = XOWRITE ( file )   
             IF answer EQ 'Y' THEN BEGIN   
                state.file=FPRAEFIX(file,/NO_PATH)                  ; Save last filename   
                state.lastpath=path                                 ; Save last path   
                extension=FEXT(file)   
                IF (extension EQ '.sav') $   
                OR (extension EQ '.SAV') THEN BEGIN   
                   SAVE,state,FILENAME=file   
                ENDIF ELSE BEGIN   
                   mess=WIDGET_MESSAGE('Extension must be SAV!',/ERROR)   
                   GOTO,exit_save_as   
                ENDELSE   
             ENDIF   
          ENDELSE   
    ENDWHILE   
exit_save_as:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
    END   
  'File.Exit': BEGIN   
    IF !D.WINDOW NE -1 THEN WDELETE                        ; Delete open window   
    WIDGET_CONTROL, event.top,  /DESTROY                   ; Destroy application   
   
    END   
  ENDCASE   
END   
   
   
   
PRO PDMENU_EDIT_Event, Event   
   
   
  CASE Event.Value OF   
   
   
  'Edit.Grid parameter': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )           ; Get child-ID   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY       ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='quake') THEN BEGIN   
       tmp=WIDGET_MESSAGE('No file open.',/ERROR)   
       GOTO,exit_parameter   
    ENDIF   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='param') THEN BEGIN   
       param=PARAM($   
            /INITIALIZE,$   
            QUAKE_LON=state.quake.lon,$   
            QUAKE_LAT=state.quake.lat,$   
            QUAKE_Z=state.quake.z, $   
            QUAKE_JS=state.quake.js)   
       IF NOT DEFINED(param,/STRUCTURE) THEN GOTO,exit_parameter   
       state=CREATE_STRUCT(state,'param',TEMPORARY(param))   
    ENDIF   
   
    param = PARAM($   
             GROUP=Group,$   
             ALTERED=state.param.altered,$   
             AREA_BORDER=state.param.area.border,$   
             CHANGES_DONE=state.param.changes_done,$   
             GRID_BORDER=state.param.grid.border,$   
             GRID_SPACING=state.param.grid.spacing,$   
             Z_SPACING=state.param.grid.z_spacing,$   
             GRID_LENGTH=state.param.grid.length,$   
             GRID_LON=state.param.grid.lon,$   
             GRID_LAT=state.param.grid.lat, $   
             GRID_Z=state.param.grid.z, $   
             GRID_DATE=state.param.grid.date, $   
             GRID_TIMEWIN=state.param.grid.timewin, $   
             GRID_SHIFT=state.param.grid.shift, $   
             QUAKE_LENGTH=state.param.quake.length,$   
             QUAKE_JS=state.quake.js,$   
             QUAKE_LAT=state.quake.lat,$   
             QUAKE_LON=state.quake.lon, $   
             QUAKE_Z=state.quake.z, $   
             QUAKE_STRENGTH=state.quake.strength, $   
             SWITCH_POINT_MODE=state.param.config.switch_point_mode,$   
             SWITCH_DEPTH_RANGE=state.param.config.switch_depth_range,$   
             SWITCH_QUAKE_NE_GRID=state.param.config.switch_quake_ne_grid, $   
             SWITCH_KEYBOARD=state.param.config.switch_keyboard,$   
             SWITCH_MOUSE=state.param.config.switch_mouse, $   
             SWITCH_NORMALIZE=state.param.config.switch_normalize, $   
             SWITCH_QUIESCENCE=state.param.config.switch_quiescence, $   
             SWITCH_ERROR=state.param.config.switch_error $   
             )   
   
    IF NOT DEFINED(param,/STRUCTURE) THEN GOTO,exit_parameter   
   
    IF param.altered NE 0 THEN BEGIN   
   
       WIDGET_CONTROL, /HOURGLASS   
   
       state=REWRITE_STRUCT(TEMPORARY(state),TAG_NAME='param',NEW_VALUE=TEMPORARY(param))   
   
    ENDIF   
   
   
exit_parameter:   
    WIDGET_CONTROL,event.handler,/SENSITIVE   
   
     WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
    END   
  'Edit.Catalog': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )           ; Get child-ID   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY       ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='quake') THEN BEGIN   
       tmp=WIDGET_MESSAGE('No file open.',/ERROR)   
       GOTO,exit_catalog   
    ENDIF   
   
    quake = EDITCAT( GROUP=Group,$   
                     ALTERED=altered,$   
                     QUAKE_JS=state.quake.js,$   
                     QUAKE_LAT=state.quake.lat,$   
                     QUAKE_LON=state.quake.lon, $   
                     QUAKE_Z=state.quake.z, $   
                     QUAKE_STRENGTH=state.quake.strength, $   
                     QUAKE_QUALITY=state.quake.quality, $   
                     QUAKE_N=state.quake.n )   
   
   
    IF NOT DEFINED(quake,/STRUCTURE) THEN GOTO,exit_catalog   
   
    WIDGET_CONTROL, /HOURGLASS   
   
    state=REWRITE_STRUCT(TEMPORARY(state),TAG_NAME='quake',NEW_VALUE=TEMPORARY(quake))   
   
   
exit_catalog:   
    WIDGET_CONTROL,event.handler,/SENSITIVE   
   
     WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
    END   
   
  'Edit.Localisation error': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )           ; Get child-ID   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY       ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='quake') THEN BEGIN   
       tmp=WIDGET_MESSAGE('No file open.',/ERROR)   
       GOTO,exit_localisation_error   
    ENDIF   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='error') THEN BEGIN   
       error = LOCERR( GROUP=Group,$   
                    /INITIALIZE,$   
                    QUAKE_QUALITY=state.quake.quality )   
       IF NOT DEFINED(error,/STRUCTURE) THEN GOTO,exit_localisation_error   
       state=CREATE_STRUCT(state,'error',TEMPORARY(error))   
    ENDIF   
   
    error = LOCERR( GROUP=Group,$   
                     ALTERED=altered,$   
                     SIGXY_JS=state.error.xy,$   
                     SIGZ=state.error.z, $   
                     SIGT=state.error.t, $   
                     QUAKE_QUALITY=state.quake.quality )   
   
   
    IF NOT DEFINED(error,/STRUCTURE) THEN GOTO,exit_localisation_error ELSE BEGIN   
   
       WIDGET_CONTROL, /HOURGLASS   
   
       state=REWRITE_STRUCT(TEMPORARY(state),TAG_NAME='error',NEW_VALUE=TEMPORARY(error))   
   
    ENDELSE   
   
   
exit_localisation_error:   
    WIDGET_CONTROL,event.handler,/SENSITIVE   
   
     WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
   
    END   
  'Edit.Result options': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )   
    ; Get state as user-value of child:   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY   
    IF NOT DEFINED(state,/STRUCTURE,TAG='quake') THEN BEGIN   
       tmp=WIDGET_MESSAGE('No file open.',/ERROR)   
       GOTO,exit_result_options   
    ENDIF   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='result') THEN BEGIN   
       result = RESULTOP ( GROUP=Group,$   
                           /INITIALIZE )   
       state=CREATE_STRUCT(state,'result',TEMPORARY(result))   
    ENDIF   
    result=RESULTOP ( GROUP=Group,$   
                      MAINTITLE=state.result.maintitle,$   
                      SUBTITLE=state.result.subtitle,$   
                      QUAKE_JS=state.quake.js,$   
                      QUAKE_LAT=state.quake.lat,$   
                      QUAKE_LON=state.quake.lon, $   
                      QUAKE_Z=state.quake.z, $   
                      QUAKE_STRENGTH=state.quake.strength, $   
                      FLAG_ANIMATION=state.result.flag_animation,$   
                      FLAG_GIF=state.result.flag_gif,$   
                      FLAG_PLOTSHOCK=state.result.flag_plotshock,$   
                      SHOCK_TO_PLOT=state.result.shock_to_plot,$   
                      COLOR=state.result.color )   
   
    IF NOT DEFINED(result,/STRUCTURE) THEN GOTO,exit_result_options ELSE BEGIN   
   
       WIDGET_CONTROL, /HOURGLASS   
   
       state=REWRITE_STRUCT(TEMPORARY(state),TAG_NAME='result',NEW_VALUE=TEMPORARY(result))   
   
    ENDELSE   
   
   
exit_result_options:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY   
    END   
  ENDCASE   
END   
   
   
   
PRO PDMENU_LAP_Event, Event   
   
   
  CASE Event.Value OF   
   
   
  'Lap.Calculate': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )   
    ; Get state as user-value of child:   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY   
    IF NOT DEFINED(state,/STRUCTURE,TAG='quake') THEN BEGIN   
       tmp=WIDGET_MESSAGE('No file open.',/ERROR)   
       GOTO,exit_calculate   
    ENDIF   
    IF NOT DEFINED(state,/STRUCTURE,TAG='param') THEN BEGIN   
       tmp=WIDGET_MESSAGE('No Grid defined.',/ERROR)   
       GOTO,exit_calculate   
    ENDIF   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='error') THEN BEGIN   
       error = LOCERR( GROUP=Group,$   
                    /INITIALIZE,$   
                    QUAKE_QUALITY=state.quake.quality )   
       IF NOT DEFINED(error,/STRUCTURE) THEN GOTO,exit_calculate   
       state=CREATE_STRUCT(state,'error',TEMPORARY(error))   
    ENDIF   
   
    n_lat= N_ELEMENTS(state.param.grid.lat)   
    n_lon= N_ELEMENTS(state.param.grid.lon)   
    n_z= N_ELEMENTS(state.param.grid.z)   
    n_epoche= LONG((state.param.grid.date(1)-state.param.grid.date(0))/state.param.grid.shift+1)   
    epoches = DBLARR(n_epoche)   
    n_seismolap= n_lat * n_lon * n_z * n_epoche   
    n_grid_elements= n_lat * n_lon * n_z   
    seismolap= DBLARR(n_seismolap)   
    s= DBLARR(n_grid_elements)   
   
    IF state.param.config.switch_quiescence THEN BEGIN   
       quiescence= seismolap   
       q = s   
    ENDIF ELSE BEGIN   
       quiescence= 0.D   
       q = 0.D   
    ENDELSE   
   
    IF state.param.config.switch_error THEN BEGIN   
       error_seismolap= seismolap   
       error_quiescence= quiescence   
       degree_freedom = LONARR(n_seismolap)   
       error_s = s   
       degree_f = LONARR(n_grid_elements)   
       error_q= q   
    ENDIF ELSE BEGIN   
       error_seismolap= 0.D   
       error_quiescence= 0.D   
       degree_freedom = 0L   
       error_s= 0.D   
       degree_f = 0L   
       error_q= 0.D   
       rel_e_s = 0.D   
       stud_t = 0.D   
    ENDELSE   
   
    n_epoches= LONG((state.param.grid.date(1)-state.param.grid.date(0))/state.param.grid.shift+1)   
    epoches = DBLARR(n_epoche)   
    epoches(0)=state.param.grid.date(0)   
    FOR t=1L,n_epoches-1L DO epoches(t)=epoches(t-1)+state.param.grid.shift   
   
   
    CASE !VERSION.OS OF   
    'Win32':   sharelib = 'slap32.dll'   
    'windows': sharelib = 'slap16.dll'   
    ELSE: sharelib = PICKFILE(PATH=state.lastpath,TITLE='Please select the sharelib')   
    ENDCASE   
   
    WIDGET_CONTROL,/HOUR   
   
    FOR t=0L,n_epoches-1L DO BEGIN   
   
        answer=CALL_EXTERNAL(sharelib, $   
                         'olap', $   
                         state.param.grid.lat, $   
                         N_ELEMENTS(state.param.grid.lat), $   
                         state.param.grid.lon, $   
                         N_ELEMENTS(state.param.grid.lon), $   
                         state.param.grid.z, $   
                         N_ELEMENTS(state.param.grid.z), $   
                         state.param.grid.length, $   
                         1L, $   
                         state.param.grid.date, $   
                         2L, $   
                         epoches(t), $   
                         N_ELEMENTS(epoches), $   
                         state.param.grid.timewin, $   
                         1L, $   
                         state.param.grid.shift, $   
                         1L, $   
                         state.quake.lat, $   
                         state.quake.n, $   
                         state.quake.lon, $   
                         state.quake.n, $   
                         state.quake.z, $   
                         state.quake.n, $   
                         state.param.quake.length, $   
                         1L, $   
                         LONG(state.quake.quality), $   
                         state.quake.n, $   
                         state.quake.js, $   
                         state.quake.n, $   
                         LONG(state.param.config.switch_normalize), $   
                         1L, $   
                         LONG(state.param.config.switch_quiescence), $   
                         1L, $   
                         LONG(state.param.config.switch_error), $   
                         1L, $   
                         state.error.xy, $   
                         N_ELEMENTS(state.error.xy), $   
                         state.error.z, $   
                         N_ELEMENTS(state.error.z), $   
                         state.error.t, $   
                         N_ELEMENTS(state.error.t), $   
                         s, $   
                         N_ELEMENTS(s), $   
                         error_s, $   
                         N_ELEMENTS(error_seismolap), $   
                         q, $   
                         N_ELEMENTS(q), $   
                         error_q, $   
                         N_ELEMENTS(error_q), $   
                         degree_f, $   
                         N_ELEMENTS(degree_f) $   
                         )   
   
   seismolap(t*n_grid_elements:(t+1)*n_grid_elements-1)=s   
   
   IF state.param.config.switch_error THEN BEGIN   
      error_seismolap(t*n_grid_elements:(t+1)*n_grid_elements-1)= error_s   
      degree_freedom(t*n_grid_elements:(t+1)*n_grid_elements-1)= degree_f   
      IF state.param.config.switch_quiescence THEN BEGIN   
         error_quiescence(t*n_grid_elements:(t+1)*n_grid_elements-1)= error_q   
      ENDIF   
   ENDIF   
   
   IF state.param.config.switch_quiescence THEN BEGIN   
      quiescence(t*n_grid_elements:(t+1)*n_grid_elements-1)= q   
   ENDIF   
   
ENDFOR   
   
IF state.param.config.switch_error THEN BEGIN   
   
   s = seismolap   
   is = WHERE( s EQ 0, count )   
   IF count NE 0 THEN s (is) = 0.01   
   
   rel_e_s = error_seismolap / s   
   
   stud_t = seismolap * 0.D   
   
   FOR i=0L,n_seismolap - 1 DO BEGIN   
   
       IF NOT ( degree_freedom ( i ) LE 1 ) THEN BEGIN   
          stud_t (i) =  ALOG10 ( T_PDF ( rel_e_s ( i ), degree_freedom ( i ) - 1 ) )   
       ENDIF   
   
   ENDFOR   
   
ENDIF   
   
   
seismolap= REFORM(seismolap,[n_lon,n_lat,n_z,n_epoche],/OVERWRITE)   
   
IF state.param.config.switch_error THEN BEGIN   
   error_seismolap= REFORM(error_seismolap,[n_lon,n_lat,n_z,n_epoche],/OVERWRITE)   
   rel_e_s = REFORM(rel_e_s,[n_lon,n_lat,n_z,n_epoche],/OVERWRITE)   
   stud_t = REFORM(stud_t,[n_lon,n_lat,n_z,n_epoche],/OVERWRITE)   
   IF state.param.config.switch_quiescence THEN BEGIN   
      error_quiescence= REFORM(error_quiescence,[n_lon,n_lat,n_z,n_epoche],/OVERWRITE)   
   ENDIF   
ENDIF   
   
   
IF state.param.config.switch_quiescence THEN BEGIN   
   quiescence= REFORM(quiescence,[n_lon,n_lat,n_z,n_epoche],/OVERWRITE)   
ENDIF   
   
   
tmp = WIDGET_MESSAGE([['Processing finished'],$   
                      ['Press Ok to save the results to disk']], $   
                      /INFORMATION)   
   
   
   
IF state.param.config.switch_normalize THEN BEGIN   
   name = 'SEISMOLAP - normalized'   
ENDIF ELSE BEGIN   
   name = 'SEISMOLAP'   
ENDELSE   
   
IF NOT DEFINED(state,/STRUCTURE,TAG='result') THEN BEGIN   
   result = RESULTOP ( GROUP=Group,$   
                       /INITIALIZE )   
   state=CREATE_STRUCT(state,'result',TEMPORARY(result))   
ENDIF   
   
lastpath = state.lastpath   
   
saved = SAV_RESULTS( 1, $   
                      state.param.grid.lat, $   
                      state.param.grid.lon, $   
                      state.param.grid.z, $   
                      state.param.grid.date, $   
                      n_lat, $   
                      n_lon, $   
                      n_z, $   
                      epoches, $   
                      state.param.grid.spacing, $   
                      state.param.grid.z_spacing, $   
                      state.param.grid.timewin, $   
                      state.param.grid.shift, $   
                      name, $   
                      state.result.maintitle, $   
                      state.result.subtitle, $   
                      TEMPORARY(seismolap), $   
                      lastpath, 's.sav')   
   
IF saved THEN state.lastpath = lastpath   
   
saved = SAV_RESULTS( state.param.config.switch_error, $   
                      state.param.grid.lat, $   
                      state.param.grid.lon, $   
                      state.param.grid.z, $   
                      state.param.grid.date, $   
                      n_lat, $   
                      n_lon, $   
                      n_z, $   
                      epoches, $   
                      state.param.grid.spacing, $   
                      state.param.grid.z_spacing, $   
                      state.param.grid.timewin, $   
                      state.param.grid.shift, $   
                      'SEISMOLAP - error', $   
                      state.result.maintitle, $   
                      state.result.subtitle, $   
                      TEMPORARY(error_seismolap), $   
                      lastpath, 's_e.sav')   
   
IF saved THEN state.lastpath = lastpath   
   
saved = SAV_RESULTS( state.param.config.switch_error, $   
                      state.param.grid.lat, $   
                      state.param.grid.lon, $   
                      state.param.grid.z, $   
                      state.param.grid.date, $   
                      n_lat, $   
                      n_lon, $   
                      n_z, $   
                      epoches, $   
                      state.param.grid.spacing, $   
                      state.param.grid.z_spacing, $   
                      state.param.grid.timewin, $   
                      state.param.grid.shift, $   
                      'SEISMOLAP - error / s', $   
                      state.result.maintitle, $   
                      state.result.subtitle, $   
                      TEMPORARY(rel_e_s), $   
                      state.lastpath, 'r_e_s.sav')   
   
IF saved THEN state.lastpath = lastpath   
   
saved = SAV_RESULTS( state.param.config.switch_error, $   
                      state.param.grid.lat, $   
                      state.param.grid.lon, $   
                      state.param.grid.z, $   
                      state.param.grid.date, $   
                      n_lat, $   
                      n_lon, $   
                      n_z, $   
                      epoches, $   
                      state.param.grid.spacing, $   
                      state.param.grid.z_spacing, $   
                      state.param.grid.timewin, $   
                      state.param.grid.shift, $   
                      'SEISMOLAP - student t - logarithmized', $   
                      state.result.maintitle, $   
                      state.result.subtitle, $   
                      TEMPORARY(stud_t), $   
                      lastpath, 'p_t.sav')   
   
IF saved THEN state.lastpath = lastpath   
   
saved = SAV_RESULTS( state.param.config.switch_quiescence, $   
                      state.param.grid.lat, $   
                      state.param.grid.lon, $   
                      state.param.grid.z, $   
                      state.param.grid.date, $   
                      n_lat, $   
                      n_lon, $   
                      n_z, $   
                      epoches, $   
                      state.param.grid.spacing, $   
                      state.param.grid.z_spacing, $   
                      state.param.grid.timewin, $   
                      state.param.grid.shift, $   
                      'SEISMOLAP - quiescence', $   
                      state.result.maintitle, $   
                      state.result.subtitle, $   
                      TEMPORARY(quiescence), $   
                      lastpath, 'q.sav')   
   
IF saved THEN state.lastpath = lastpath   
   
IF state.param.config.switch_quiescence THEN BEGIN   
   
   saved = SAV_RESULTS( state.param.config.switch_error, $   
                      state.param.grid.lat, $   
                      state.param.grid.lon, $   
                      state.param.grid.z, $   
                      state.param.grid.date, $   
                      n_lat, $   
                      n_lon, $   
                      n_z, $   
                      epoches, $   
                      state.param.grid.spacing, $   
                      state.param.grid.z_spacing, $   
                      state.param.grid.timewin, $   
                      state.param.grid.shift, $   
                      'SEISMOLAP - error of quiescence', $   
                      state.result.maintitle, $   
                      state.result.subtitle, $   
                      TEMPORARY(error_quiescence), $   
                      lastpath, $   
                      'q_e.sav')   
   
IF saved THEN state.lastpath = lastpath   
   
ENDIF   
   
   
exit_calculate:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY   
    END   
'Lap.Calculate probability': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )          ; Get child-ID   
   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY      ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='lastpath') THEN BEGIN   
       state=CREATE_STRUCT('lastpath','')   
       lastpath=''   
    ENDIF ELSE lastpath=state.lastpath   
   
    file = PICKFILE(TITLE='Open File',$                   ; Get name of data-file   
                     FILTER='*.sav',$   
                      PATH=lastpath,$   
                       GET_PATH=lastpath)   
    IF file EQ '' THEN GOTO, exit_probability                 ; 'Cancel' was pressed   
   
    state.lastpath=lastpath   
   
    RESTORE,file   
   
    WIDGET_CONTROL,/HOUR   
    probability = data * 0   
    n_epoches = N_ELEMENTS(epoches)   
    prob = DBLARR(n_epoches)   
    FOR k=0L,n_z - 1 DO BEGIN   
        FOR i=0L,n_lon - 1 DO BEGIN   
            FOR j=0L,n_lat - 1 DO BEGIN   
                PPEARS3,data(i,j,k,*)*(-1.),timewin,shift,prob   
                probability(i,j,k,*) = prob   
            ENDFOR   
        ENDFOR   
    ENDFOR   
   
    tmp = WIDGET_MESSAGE([['Processing finished'],$   
                         ['Press Ok to save the result to disk']], $   
                         /INFORMATION )   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='result') THEN BEGIN   
       result = RESULTOP ( GROUP=Group,$   
                       /INITIALIZE )   
       state=CREATE_STRUCT(state,'result',TEMPORARY(result))   
    ENDIF   
   
    saved = SAV_RESULTS( 1, $   
                      lat, $   
                      lon, $   
                      z, $   
                      t, $   
                      n_lat, $   
                      n_lon, $   
                      n_z, $   
                      epoches, $   
                      spacing, $   
                      z_spacing, $   
                      timewin, $   
                      shift, $   
                      'Probability Pearson Type 3', $   
                      maintitle, $   
                      subtitle, $   
                      TEMPORARY(probability), $   
                      lastpath, $   
                      'p_p3.sav')   
   
IF saved THEN state.lastpath = lastpath   
   
   
exit_probability:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
   
    END   
'Lap.Find value.Maximum': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )          ; Get child-ID   
   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY      ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='lastpath') THEN BEGIN   
       state=CREATE_STRUCT('lastpath','', $   
                           'file','')   
       lastpath=''   
    ENDIF ELSE lastpath=state.lastpath   
   
    answer = FIND_EXTREMUM ( /MAX, PATH=lastpath )   
   
    IF answer THEN state.lastpath = lastpath   
   
exit_search_maximum:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
    END   
   
'Lap.Find value.Minimum': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )          ; Get child-ID   
   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY      ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='lastpath') THEN BEGIN   
       state=CREATE_STRUCT('lastpath','', $   
                           'file','')   
       lastpath=''   
    ENDIF ELSE lastpath=state.lastpath   
   
    answer = FIND_EXTREMUM ( /MIN, PATH=lastpath )   
   
    IF answer THEN state.lastpath = lastpath   
   
exit_search_minimum:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
   
    END   
'Lap.Find value.by Date': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )          ; Get child-ID   
   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY      ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='lastpath') THEN BEGIN   
       state=CREATE_STRUCT('lastpath','', $   
                           'file','')   
       lastpath=''   
    ENDIF ELSE lastpath=state.lastpath   
   
    answer = FIND_DATUM ( PATH=lastpath )   
   
    IF answer THEN state.lastpath = lastpath   
   
exit_search_datum:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
   
    END   
  ENDCASE   
END   
   
   
   
PRO PDMENU_VIEW_Event, Event   
   
   
  CASE Event.Value OF   
   
   
  'View.Start map': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )           ; Get child-ID   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY       ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='quake') THEN BEGIN            ; If state is not a structure   
       tmp=WIDGET_MESSAGE('No File open.',/ERROR)   ; No file open   
   
       GOTO,exit_map   
    ENDIF ELSE BEGIN   
       WIDGET_CONTROL, event.handler, GET_UVALUE=xqbase,/NO_COPY       ; Get state as user-value of child   
   
   
    SAV_DIS,saved_system   
   
    CASE !VERSION.OS OF   
    'Win32':   hires = 1   
    'windows': hires = 0   
    ELSE: hires = 1   
    ENDCASE   
       IF DEFINED(xqbase) THEN BEGIN   
          XQUAKE,state.quake.lat,$   
              state.quake.lon,$   
              /LABEL,$   
              /GRID,$   
              HIRES = hires,$   
              TITLE='Map'   
       ENDIF   
    ENDELSE   
exit_map:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
    RES_DIS,saved_system   
    END   
   
  'View.Statistics': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )   
    ; Get state as user-value of child:   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY   
    IF NOT DEFINED(state,/STRUCTURE,TAG='quake') THEN BEGIN   
       tmp=WIDGET_MESSAGE('No file open.',/ERROR)   
       GOTO,exit_stat   
    ENDIF   
   
    XSTAT, QUAKE_MAG=state.quake.strength,$   
           QUAKE_Z=state.quake.z,$   
           QUAKE_JS=state.quake.js,$   
           QUAKE_QUALITY=state.quake.quality   
   
   
exit_stat:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY   
    END   
   
  'View.Results': BEGIN   
    child = WIDGET_INFO( event.handler, /CHILD )          ; Get child-ID   
   
    WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY      ; Get state as user-value of child   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='lastpath') THEN BEGIN   
       state=CREATE_STRUCT('lastpath','')   
       lastpath=''   
    ENDIF ELSE lastpath=state.lastpath   
   
    file = PICKFILE(TITLE='Open File',$                   ; Get name of data-file   
                     FILTER='*.sav',$   
                      PATH=lastpath,$   
                       GET_PATH=path)   
    IF file EQ '' THEN GOTO, exit_results                 ; 'Cancel' was pressed   
   
    state.lastpath=path   
   
    IF NOT (STRUPCASE(FEXT(file,/NOPOINT)) EQ 'SAV') THEN BEGIN   
       XDISPLAYFILE,file   
       GOTO, exit_results   
    ENDIF   
   
    RESTORE,file   
   
    IF NOT DEFINED(state,/STRUCTURE,TAG='result') THEN BEGIN   
       result = RESULTOP ( GROUP=Group,$   
                       /INITIALIZE )   
       state=CREATE_STRUCT(state,'result',TEMPORARY(result))   
    ENDIF   
   
    IF NOT DEFINED(im) THEN BEGIN   
   
       IF NOT DEFINED(data) THEN BEGIN   
          mess=WIDGET_MESSAGE('No data!',/ERROR)   
          GOTO,exit_results   
       ENDIF   
   
       im=VIEW_RESULTS(GROUP=Group,$   
                 SIZE_N=[n_lat,n_lon,n_z],$   
                 LON=lon,$   
                 LAT=lat,$   
                 DEPTH= z,$   
                 GRID_DATE=t,$   
                 DATA=TEMPORARY(data),$   
                 EPOCHE=TEMPORARY(epoches),$   
                 GRID_TIMEWIN=timewin,$   
                 GRID_SHIFT=shift,$   
                 MAINTITLE=maintitle, $   
                 SUBTITLE=subtitle, $   
                 COLOR=state.result.color,$   
                 FILE=state.lastpath,$   
                 PATH=state.lastpath,$   
                 WRITE_GIF=state.result.flag_gif,$   
                 ANIMATION=state.result.flag_animation,$   
                 PLOTSHOCK=state.result.flag_plotshock,$   
                 SHOCK_TO_PLOT=state.result.shock_to_plot,$   
                 NAME=name $   
                 )   
   
       IF N_ELEMENTS(im) EQ 1 THEN GOTO,exit_results   
   
       tmp = WIDGET_MESSAGE([['Processing finished'],$   
                         ['Press Ok to save the animation to disk']], $   
                         /INFORMATION )   
   
       answer = 'N'   
   
       WHILE answer EQ 'N' DO BEGIN   
   
          file = PICKFILE(/WRITE, $   
                          PATH=state.lastpath, $   
                          FILE=state.lastpath+'an_'+STRTRUNC(FPRAEFIX(file,/NO_PATH),5,0)+'.sav', $   
                          FILTER='*.sav',$   
                          GET_PATH=path)   
   
          IF file EQ '' THEN GOTO, exit_results                 ; 'Cancel' was pressed   
   
          answer = XOWRITE ( file )   
   
          IF answer EQ 'Y' THEN BEGIN   
   
             state.lastpath=path   
   
             SAVE,FILENAME=file,im   
   
          ENDIF   
   
       ENDWHILE   
   
    ENDIF ELSE BEGIN   
   
       LOADCT, state.result.color   
       epoche_n=N_ELEMENTS(im(0,0,*))   
       XINTERANIMATE, SET=[N_ELEMENTS(im(*,0)),N_ELEMENTS(im(0,*)),epoche_n], /SHOWLOAD   
       FOR t=0,epoche_n-1 DO XINTERANIMATE, FRAME = t, IMAGE = im(*,*,t)   
       XINTERANIMATE   
   
    ENDELSE   
   
exit_results:   
    WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
    END   
  ENDCASE   
END   
   
   
   
   
PRO PDMENU_HELP_Event, Event   
   
   
  CASE Event.Value OF   
   
   
  'Help.About Slap...': BEGIN   
    tmp=WIDGET_MESSAGE(TITLE='About this program', $   
                        ['SLAP, Version 1.0',$   
                        'Author: Matthias Liebetanz, Darmstadt University of Technology, Germany.',$   
                        'The Author makes no warranty that this program is free of error.'],$   
                        /INFORMATION )   
   END   
  ENDCASE   
END   
   
   
   
   
PRO MAIN13_Event, Event   
   
   
  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev   
   
  CASE Ev OF   
   
  'PDMENU_FILE': PDMENU_FILE_Event, Event   
  'PDMENU_EDIT': PDMENU_EDIT_Event, Event   
  'PDMENU_LAP': PDMENU_LAP_Event, Event   
  'PDMENU_VIEW': PDMENU_VIEW_Event, Event   
  'PDMENU_HELP': PDMENU_HELP_Event, Event   
  ENDCASE   
END   
   
   
   
   
PRO SLAP, GROUP=Group   
   
   
  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0   
   
  junk   = { CW_PDMENU_S, flags:0, name:'' }   
   
   
  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $   
      ROW=1, $   
      MAP=1, $   
      TITLE='Slap', $   
      UVALUE='MAIN13')   
   
  BASE47 = WIDGET_BASE(MAIN13, $   
      ROW=1, $   
      MAP=0, $   
      UVALUE='BASE47')   
   
   
  MenuDesc891 = [ $   
      { CW_PDMENU_S,       3, 'File' }, $ ;        0   
        { CW_PDMENU_S,       0, 'Close' }, $ ;        1   
        { CW_PDMENU_S,       0, 'Open' }, $ ;        2   
        { CW_PDMENU_S,       0, 'Save as' }, $ ;        3   
        { CW_PDMENU_S,       2, 'Exit' } $  ;      4   
   
  ]   
   
   
  PDMENU_FILE = CW_PDMENU( MAIN13, MenuDesc891, /RETURN_FULL_NAME, $   
      UVALUE='PDMENU_FILE')   
   
  MenuDesc1218 = [ $   
      { CW_PDMENU_S,       3, 'Edit' }, $ ;        0   
        { CW_PDMENU_S,       0, 'Grid parameter' }, $ ;        1   
        { CW_PDMENU_S,       0, 'Catalog' }, $ ;        2   
        { CW_PDMENU_S,       0, 'Localisation error' }, $ ;        3   
        { CW_PDMENU_S,       2, 'Result options' } $  ;      4   
   
  ]   
   
   
  PDMENU_EDIT = CW_PDMENU( MAIN13, MenuDesc1218, /RETURN_FULL_NAME, $   
      UVALUE='PDMENU_EDIT')   
   
  MenuDesc975 = [ $   
      { CW_PDMENU_S,       3, 'Lap' }, $ ;        0   
        { CW_PDMENU_S,       0, 'Calculate' }, $ ;        1   
        { CW_PDMENU_S,       0, 'Calculate probability' }, $ ;        2   
        { CW_PDMENU_S,       3, 'Find value' }, $ ;        3   
          { CW_PDMENU_S,       0, 'Maximum' }, $ ;        4   
          { CW_PDMENU_S,       0, 'Minimum' }, $ ;        5   
          { CW_PDMENU_S,       2, 'by Date' } $  ;      6   
   
  ]   
   
   
  PDMENU_LAP = CW_PDMENU( MAIN13, MenuDesc975, /RETURN_FULL_NAME, $   
      UVALUE='PDMENU_LAP')   
   
  MenuDesc1325 = [ $   
      { CW_PDMENU_S,       3, 'View' }, $ ;        0   
        { CW_PDMENU_S,       0, 'Start map' }, $ ;        1   
        { CW_PDMENU_S,       0, 'Statistics' }, $ ;        2   
        { CW_PDMENU_S,       2, 'Results' } $  ;      3   
   
  ]   
   
   
  PDMENU_VIEW = CW_PDMENU( MAIN13, MenuDesc1325, /RETURN_FULL_NAME, $   
      UVALUE='PDMENU_VIEW')   
   
  MenuDesc1441 = [ $   
      { CW_PDMENU_S,       3, 'Help' }, $ ;        0   
        { CW_PDMENU_S,       2, 'About Slap...' } $  ;      1   
   
  ]   
   
   
  PDMENU_HELP = CW_PDMENU( MAIN13, MenuDesc1441, /RETURN_FULL_NAME, $   
      UVALUE='PDMENU_HELP')   
   
  state=CREATE_STRUCT(' ',' ')                                              ; Initialize state   
   
  child = WIDGET_INFO( MAIN13, /CHILD )                     ; Get child-ID   
   
  WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Set state   
   
  WIDGET_CONTROL, MAIN13, /REALIZE   
   
  XMANAGER, 'MAIN13', MAIN13   
END   
