;-------------------------------------------------------------
;+
; NAME:
;       SPATIALWIN
; PURPOSE:
;       Sets spatial parameters
; MAIN PROGRAM:
;       SLAP - program for testing SEISMOLAP
;       SEISMOLAP was developed by Jochen Zschau,
;       Geoforschungszentrum Potsdam, Germany
; MODIFICATION HISTORY:
;       Matthias Liebetanz, Darmstadt University of Technology, Germany, 19 May 1998.
;
;-
;-------------------------------------------------------------
PRO SPATIALWIN, GROUP=Group,$
            INITIALIZE=INITIALIZE,$
            ALTERED=altered,$
            AREA_BORDER=area_border,$
            GRID_BORDER=grid_border,$
            GRID_LON=grid_lon,$
            GRID_LAT=grid_lat,$
            GRID_Z=grid_z,$
            GRID_SPACING=grid_spacing,$
            Z_SPACING=z_spacing,$
            GRID_LENGTH=grid_length,$
            QUAKE_LON=quake_lon,$
            QUAKE_LAT=quake_lat,$
            QUAKE_Z=quake_z,$
            QUAKE_LENGTH=quake_length,$
            SWITCH_QUAKE_NE_GRID=switch_quake_ne_grid,$
            SWITCH_POINT_MODE=switch_point_mode,$
            SWITCH_DEPTH_RANGE=switch_depth_range,$
            SWITCH_KEYBOARD=switch_keyboard,$
            SWITCH_MOUSE=switch_mouse,$
            SWITCH_NORMALIZE=switch_normalize,$
            SWITCH_QUIESCENCE=switch_quiescence,$
            SWITCH_ERROR=switch_error


  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

  IF KEYWORD_SET(INITIALIZE) THEN BEGIN

     grid_border=DBLARR(6)
     grid_border(0)=MIN(quake_lon,MAX=tmp)
     grid_border(1)=tmp
     grid_border(2)=MIN(quake_lat,MAX=tmp)
     grid_border(3)=tmp
     grid_border(4)=MIN(quake_z,MAX=tmp)
     grid_border(5)=tmp

     area_border=grid_border

     grid_length=5000.D

     grid_lon=DBLARR(1,2)
     grid_lat=DBLARR(1,2)
     grid_z=DBLARR(1,2)

     grid_spacing=0.05
     z_spacing=1000.D
     MORE_SPATIAL, $
            /INITIALIZE,$
            ALTERED=altered,$
            SWITCH_KEYBOARD=switch_keyboard,$
            SWITCH_MOUSE=switch_mouse,$
            SWITCH_POINT_MODE=switch_point_mode,$
            SWITCH_DEPTH_RANGE=switch_depth_range,$
            SWITCH_QUAKE_NE_GRID=switch_quake_ne_grid,$
            SWITCH_NORMALIZE=switch_normalize,$
            SWITCH_QUIESCENCE=switch_quiescence,$
            SWITCH_ERROR=switch_error

     CASE switch_point_mode OF
         1: BEGIN
            WIDGET_CONTROL,BASE278,SENSITIVE=0
            area_border(0:1)=area_border(0)
            area_border(2:3)=area_border(2)
            area_border(4:5)=area_border(4)
            END
         0: BEGIN
            END
     ENDCASE

     CASE switch_depth_range OF
            0: BEGIN
               area_border(5)=area_border(4)
               END
            1: BEGIN
               END
     ENDCASE


     CASE switch_quake_ne_grid OF
         0: BEGIN
               quake_length=grid_length
            END
         1: BEGIN
               quake_length=grid_length
            END
     ENDCASE


     GRIDINIT,grid_spacing,z_spacing,area_border,grid_lon,grid_lat,grid_z

     altered=0

     RETURN

  ENDIF

  ;init borders
  lon_min = MIN ( quake_lon, MAX = lon_max )
  lat_min = MIN ( quake_lat, MAX = lat_max )
  z_min = MIN ( quake_z, MAX = z_max )

  IF lon_min GT grid_border(0) THEN grid_border(0) = lon_min
  IF lon_max LT grid_border(1) THEN grid_border(1) = lon_max
  IF lat_min GT grid_border(2) THEN grid_border(2) = lat_min
  IF lat_max LT grid_border(3) THEN grid_border(3) = lat_max
  IF z_min GT grid_border(4) THEN grid_border(4) = z_min
  IF z_max LT grid_border(5) THEN grid_border(5) = z_max

  GRIDINIT,grid_spacing,z_spacing,area_border,grid_lon,grid_lat,grid_z

  ;Saving initial values - CANCEL may be pressed
  area_border_old=area_border
  grid_spacing_old=grid_spacing
  z_spacing_old=z_spacing
  grid_length_old=grid_length
  grid_lon_old=grid_lon
  grid_lat_old=grid_lat
  grid_z_old=grid_z
  switch_point_mode_old=switch_point_mode
  switch_depth_range_old=switch_depth_range
  switch_quake_ne_grid_old=switch_quake_ne_grid
  switch_keyboard_old=switch_keyboard
  switch_mouse_old=switch_mouse
  switch_normalize_old=switch_normalize
  switch_quiescence_old=switch_quiescence
  switch_error_old=switch_error

  junk   = { CW_PDMENU_S, flags:0, name:'' }


  MAIN1 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, $
      TITLE='Spatial window', $
      UVALUE='MAIN1')

  BASE28 = WIDGET_BASE(MAIN1, $
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE28')

  BASE47 = WIDGET_BASE(BASE28, $
      COLUMN=1, $
      FRAME=1, $
      MAP=1, $
      UVALUE='BASE47')

  LABEL34 = WIDGET_LABEL( BASE47, $
      UVALUE='LABEL34', $
      VALUE='Grid')

  BASE158 = WIDGET_BASE(BASE47, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE158')

  LABEL35 = WIDGET_LABEL( BASE158, $
      UVALUE='LABEL35', $
      VALUE='Longitude...')

  FieldVal4104 = [ $
    '' ]
  FIELD_LON_MIN = CW_FIELD( BASE158,VALUE=FieldVal4104, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='min', $
      UVALUE='FIELD_LON_MIN', $
      XSIZE=9)

  FieldVal4106 = [ $
    ' ' ]
  FIELD_LON_MAX = CW_FIELD( BASE158,VALUE=FieldVal4106, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='max', $
      UVALUE='FIELD_LON_MAX', $
      XSIZE=9)


  BASE152 = WIDGET_BASE(BASE47, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE152')

  LABEL38 = WIDGET_LABEL( BASE152, $
      UVALUE='LABEL38', $
      VALUE='Latitude......')

  FieldVal4110 = [ $
    '' ]
  FIELD_LAT_MIN = CW_FIELD( BASE152,VALUE=FieldVal4110, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='min', $
      UVALUE='FIELD_LAT_MIN', $
      XSIZE=9)

  FieldVal4112 = [ $
    ' ' ]
  FIELD_LAT_MAX = CW_FIELD( BASE152,VALUE=FieldVal4112, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='max', $
      UVALUE='FIELD_LAT_MAX', $
      XSIZE=9)


  BUTTON_SET_COORD_WITH_MOUSE = WIDGET_BUTTON( BASE47, $
      UVALUE='BUTTON_SET_COORD_WITH_MOUSE', $
      VALUE='Set coordinates with mouse')

  BASE260 = WIDGET_BASE(BASE47, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE260')

  LABEL257 = WIDGET_LABEL( BASE260, $
      UVALUE='LABEL257', $
      VALUE='Depth [m]..')

  FieldVal4116 = [ $
    '' ]
  FIELD_DEP_MIN = CW_FIELD( BASE260,VALUE=FieldVal4116, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='min', $
      UVALUE='FIELD_DEP_MIN', $
      XSIZE=9)

  FieldVal4118 = [ $
    ' ' ]
  FIELD_DEP_MAX = CW_FIELD( BASE260,VALUE=FieldVal4118, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='max', $
      UVALUE='FIELD_DEP_MAX', $
      XSIZE=9)


  LABEL293 = WIDGET_LABEL( BASE47, $
      UVALUE='LABEL293', $
      VALUE='')

  FieldVal4120 = [ $
    '' ]
  FIELD_GRID_LENGTH = CW_FIELD( BASE47,VALUE=FieldVal4120, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='Cube length [m]', $
      UVALUE='FIELD_GRID_LENGTH', $
      XSIZE=9)

  BASE278 = WIDGET_BASE(BASE47, $
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE278')

  FieldVal4212 = [ $
    '' ]
  FIELD_GRID_SPACING = CW_FIELD( BASE278,VALUE=FieldVal4212, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='Grid spacing [deg]', $
      UVALUE='FIELD_GRID_SPACING', $
      XSIZE=9)

  FieldVal4210 = [ $
    '' ]
  FIELD_Z_SPACING = CW_FIELD( BASE278,VALUE=FieldVal4210, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='Z spacing [m]', $
      UVALUE='FIELD_Z_SPACING', $
      XSIZE=9)

  BUTTON_SHOW_GRID = WIDGET_BUTTON( BASE278, $
      UVALUE='BUTTON_SHOW_GRID', $
      VALUE='Show grid')

  BASE33 = WIDGET_BASE(BASE28, $
      COLUMN=1, $
      FRAME=1, $
      MAP=1, $
      UVALUE='BASE33')

  LABEL95 = WIDGET_LABEL( BASE33, $
      UVALUE='LABEL95', $
      VALUE='Earthquakes')

  FieldVal4130 = [ $
    '' ]
  FIELD_QUAKE_LENGTH = CW_FIELD( BASE33,VALUE=FieldVal4130, $
      ROW=1, $
      FLOAT=1, $
      ALL_EVENTS=1, $
      TITLE='Cube length [m]', $
      UVALUE='FIELD_QUAKE_LENGTH', $
      XSIZE=9)

  LABEL666 = WIDGET_LABEL( MAIN1, $
      UVALUE='LABEL666', $
      VALUE='')

  BUTTON_MORE_SPATIAL = WIDGET_BUTTON( MAIN1, $
      UVALUE='BUTTON_MORE_SPATIAL', $
      VALUE='More settings')

  LABEL256 = WIDGET_LABEL( MAIN1, $
      UVALUE='LABEL256', $
      VALUE='')

  BUTTON_CANCEL = WIDGET_BUTTON( MAIN1, $
      UVALUE='BUTTON_CANCEL', $
      VALUE='Cancel')

  BUTTON_OK = WIDGET_BUTTON( MAIN1, $
      UVALUE='BUTTON_OK', $
      VALUE='Ok')

  WIDGET_CONTROL,FIELD_LON_MIN,SET_VALUE=area_border(0)
  WIDGET_CONTROL,FIELD_LAT_MIN,SET_VALUE=area_border(2)
  WIDGET_CONTROL,FIELD_DEP_MIN,SET_VALUE=area_border(4)
  WIDGET_CONTROL,FIELD_LON_MAX,SET_VALUE=area_border(1)
  WIDGET_CONTROL,FIELD_LAT_MAX,SET_VALUE=area_border(3)
  WIDGET_CONTROL,FIELD_DEP_MAX,SET_VALUE=area_border(5)
  WIDGET_CONTROL,FIELD_GRID_SPACING,SET_VALUE=grid_spacing
  WIDGET_CONTROL,FIELD_Z_SPACING,SET_VALUE=z_spacing
  WIDGET_CONTROL,FIELD_GRID_LENGTH,SET_VALUE=grid_length
  WIDGET_CONTROL,FIELD_QUAKE_LENGTH,SET_VALUE=quake_length
  IF NOT switch_keyboard THEN BEGIN
      WIDGET_CONTROL,BASE158,SENSITIVE=0
      WIDGET_CONTROL,BASE152,SENSITIVE=0
  ENDIF
  IF switch_point_mode THEN BEGIN
     WIDGET_CONTROL,BASE278,SENSITIVE=0
  ENDIF

  WIDGET_CONTROL, MAIN1, /REALIZE

Ev=''

WHILE (Ev NE 'BUTTON_OK') AND (Ev NE 'BUTTON_CANCEL') DO BEGIN

Event=WIDGET_EVENT(MAIN1)

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  CASE Ev OF

  'FIELD_LON_MIN': BEGIN
      WIDGET_CONTROL,FIELD_LON_MIN,GET_VALUE=tmp
      area_border(0)=tmp
      IF switch_point_mode THEN BEGIN
         area_border(0)=tmp
         area_border(1)=area_border(0)
         WIDGET_CONTROL,FIELD_LON_MAX,SET_VALUE=area_border(1)
      ENDIF
      altered=1
      END
  'FIELD_LON_MAX': BEGIN
      WIDGET_CONTROL,FIELD_LON_MAX,GET_VALUE=tmp
      area_border(1)=tmp
      IF switch_point_mode THEN BEGIN
         area_border(1)=tmp
         area_border(0)=area_border(1)
         WIDGET_CONTROL,FIELD_LON_MIN,SET_VALUE=area_border(0)
      ENDIF
      altered=1
      END
  'FIELD_LAT_MIN': BEGIN
      WIDGET_CONTROL,FIELD_LAT_MIN,GET_VALUE=tmp
      area_border(2)=tmp
      IF switch_point_mode THEN BEGIN
         area_border(2)=tmp
         area_border(3)=area_border(2)
         WIDGET_CONTROL,FIELD_LAT_MAX,SET_VALUE=area_border(3)
      ENDIF
      altered=1
      END
  'FIELD_LAT_MAX': BEGIN
      WIDGET_CONTROL,FIELD_LAT_MAX,GET_VALUE=tmp
      area_border(3)=tmp
      IF switch_point_mode THEN BEGIN
         area_border(2)=area_border(3)
         WIDGET_CONTROL,FIELD_LAT_MIN,SET_VALUE=area_border(2)
      ENDIF
      altered=1
      END
  'FIELD_DEP_MIN': BEGIN
      WIDGET_CONTROL,FIELD_DEP_MIN,GET_VALUE=tmp
      area_border(4)=tmp
      CASE switch_depth_range OF
      0: BEGIN
         area_border(5)=area_border(4)
         WIDGET_CONTROL,FIELD_DEP_MAX,SET_VALUE=area_border(5)
         altered=1
         END
      1: BEGIN
         END
      ENDCASE
      END
  'FIELD_DEP_MAX': BEGIN
      WIDGET_CONTROL,FIELD_DEP_MAX,GET_VALUE=tmp
      area_border(5)=tmp
      CASE switch_depth_range OF
      0: BEGIN
         area_border(4)=area_border(5)
         WIDGET_CONTROL,FIELD_DEP_MIN,SET_VALUE=area_border(4)
         altered=1
         END
      1: BEGIN
         END
      ENDCASE
      END
  'FIELD_GRID_SPACING': BEGIN
      WIDGET_CONTROL,FIELD_GRID_SPACING,GET_VALUE=value
      IF (value LE 0.) THEN BEGIN
         mess=WIDGET_MESSAGE('Must be greater than 0',/ERROR)
         WIDGET_CONTROL,FIELD_GRID_SPACING,SET_VALUE=grid_spacing
      ENDIF ELSE BEGIN
         grid_spacing=value
         GRIDINIT,grid_spacing,z_spacing,area_border,grid_lon,grid_lat,grid_z
      ENDELSE
      altered=1
      END
  'FIELD_GRID_LENGTH': BEGIN
      WIDGET_CONTROL,FIELD_GRID_LENGTH,GET_VALUE=grid_length
      IF grid_length LT 0.D THEN BEGIN
         grid_length=0.D
         WIDGET_CONTROL,FIELD_GRID_LENGTH,SET_VALUE=grid_length
      ENDIF
      IF NOT switch_quake_ne_grid THEN BEGIN
         quake_length=grid_length
         WIDGET_CONTROL,FIELD_QUAKE_LENGTH,SET_VALUE=grid_length
      ENDIF

      altered=1
      END
  'FIELD_QUAKE_LENGTH': BEGIN
      IF switch_quake_ne_grid THEN BEGIN
         WIDGET_CONTROL,FIELD_QUAKE_LENGTH,GET_VALUE=quake_length
         altered=1
      ENDIF ELSE BEGIN
         WIDGET_CONTROL,FIELD_QUAKE_LENGTH,SET_VALUE=grid_length
      ENDELSE
      END
  'FIELD_Z_SPACING': BEGIN
      WIDGET_CONTROL,FIELD_Z_SPACING,GET_VALUE=tmp
      WIDGET_CONTROL,/HOUR
      IF (tmp LE 0) THEN BEGIN
          mess=WIDGET_MESSAGE('Must be greater than 0.',/ERROR)
          z_spacing=1.
          WIDGET_CONTROL,FIELD_Z_SPACING,SET_VALUE=z_spacing
      ENDIF ELSE BEGIN
          z_spacing=tmp
          GRIDINIT,grid_spacing,z_spacing,area_border,grid_lon,grid_lat,grid_z
      ENDELSE
      altered=1

      END
  'BUTTON_SET_COORD_WITH_MOUSE': BEGIN
      CASE switch_mouse OF
      0: BEGIN
         END
      1: BEGIN
          tmp1=grid_border(0:3)
          tmp2=area_border(0:3)

          gridar=GRIDAREA(quake_lon,$
                          quake_lat,$
                          PLOTPOS=plotpos,$
                          POINT_MODE=switch_point_mode, $
                          LIMITS=tmp1,$
                          AREA=tmp2)

          grid_border(0:3)=tmp1
          area_border(0:3)=tmp2

          IF gridar(0) NE -1 THEN BEGIN
             area_border(0:3)=gridar
             WIDGET_CONTROL,FIELD_LON_MIN,SET_VALUE=area_border(0)
             WIDGET_CONTROL,FIELD_LON_MAX,SET_VALUE=area_border(1)
             WIDGET_CONTROL,FIELD_LAT_MIN,SET_VALUE=area_border(2)
             WIDGET_CONTROL,FIELD_LAT_MAX,SET_VALUE=area_border(3)
          ENDIF

          altered=1
         END
      ENDCASE

      END
  'BUTTON_SHOW_GRID': BEGIN

      GRIDINIT,grid_spacing,z_spacing,area_border,grid_lon,grid_lat,grid_z

      tmp=grid_border(0:3)

      gridar=GRIDAREA(quake_lon,$
                          quake_lat,$
                          PLOTPOS=plotpos,$
                          LIMITS=tmp,$
                          GRID_X=grid_lon,$
                          GRID_Y=grid_lat,$
                          /SUPRESS_SELECT)

      grid_border(0:3)=tmp

      END
  'BUTTON_MORE_SPATIAL': BEGIN

      MORE_SPATIAL,$
            ALTERED=altered,$
            SWITCH_KEYBOARD=switch_keyboard,$
            SWITCH_MOUSE=switch_mouse,$
            SWITCH_POINT_MODE=switch_point_mode,$
            SWITCH_DEPTH_RANGE=switch_depth_range,$
            SWITCH_QUAKE_NE_GRID=switch_quake_ne_grid,$
            SWITCH_NORMALIZE=switch_normalize,$
            SWITCH_QUIESCENCE=switch_quiescence,$
            SWITCH_ERROR=switch_error


      IF altered THEN BEGIN

         ; Changes were made

         CASE switch_point_mode OF
         1: BEGIN
            area_border(0:1)=area_border(0)
            area_border(2:3)=area_border(2)
            area_border(4:5)=area_border(4)
            WIDGET_CONTROL,FIELD_LON_MIN,SET_VALUE=area_border(0)
            WIDGET_CONTROL,FIELD_LAT_MIN,SET_VALUE=area_border(2)
            WIDGET_CONTROL,FIELD_DEP_MIN,SET_VALUE=area_border(4)
            WIDGET_CONTROL,FIELD_LON_MAX,SET_VALUE=area_border(1)
            WIDGET_CONTROL,FIELD_LAT_MAX,SET_VALUE=area_border(3)
            WIDGET_CONTROL,FIELD_DEP_MAX,SET_VALUE=area_border(5)
            WIDGET_CONTROL,BASE278,SENSITIVE=0
            END
         0: BEGIN
            WIDGET_CONTROL,BASE278,/SENSITIVE
            END
         ENDCASE

         CASE switch_depth_range OF
            0: BEGIN
               area_border(5)=area_border(4)
               WIDGET_CONTROL,FIELD_DEP_MAX,SET_VALUE=area_border(5)
               END
            1: BEGIN
               END
         ENDCASE

         CASE switch_keyboard OF
            0: BEGIN
               WIDGET_CONTROL,BASE158,SENSITIVE=0
               WIDGET_CONTROL,BASE152,SENSITIVE=0
               END
            1: BEGIN
               WIDGET_CONTROL,BASE158,/SENSITIVE
               WIDGET_CONTROL,BASE152,/SENSITIVE
               END
         ENDCASE

         CASE switch_quake_ne_grid OF
         0: BEGIN
            quake_length=grid_length
            WIDGET_CONTROL,FIELD_QUAKE_LENGTH,SET_VALUE=quake_length
            END
         1: BEGIN
            END
         ENDCASE

      ENDIF
      END
  'BUTTON_CANCEL': BEGIN
      END
  'BUTTON_OK': BEGIN
      END
  ENDCASE

ENDWHILE

IF Ev EQ 'BUTTON_OK' THEN BEGIN

   IF altered THEN BEGIN

      GRIDINIT,grid_spacing,z_spacing,area_border,grid_lon,grid_lat,grid_z

      grid_length=DOUBLE(grid_length)
      quake_length=DOUBLE(quake_length)

   ENDIF

ENDIF ELSE BEGIN     ;Cancel
      ;Restoring initial values
      area_border=area_border_old
      grid_spacing=grid_spacing_old
      z_spacing=z_spacing_old
      grid_length=grid_length_old
      grid_lon=grid_lon_old
      grid_lat=grid_lat_old
      grid_z=grid_z_old
      switch_point_mode=switch_point_mode_old
      switch_depth_range=switch_depth_range_old
      switch_quake_ne_grid=switch_quake_ne_grid_old
      switch_keyboard=switch_keyboard_old
      switch_mouse=switch_mouse_old
      switch_normalize=switch_normalize_old
      switch_quiescence=switch_quiescence_old
      switch_error=switch_error_old

      ;Setting flag
      altered=0
ENDELSE

IF DEFINED(MAIN1) THEN WIDGET_CONTROL,MAIN1,/DESTROY

END
