;------------------------------------------------------------- 
;+ 
; NAME: 
;       VIEW_RESULTS 
; PURPOSE: 
;       Subroutine for visualization SEISMOLAP - data. 
; MAIN PROGRAM: 
;       SLAP - program for testing SEISMOLAP 
;       SEISMOLAP was developed by Jochen Zschau, 
;       Geoforschungszentrum Potsdam, Germany 
; MODIFICATION HISTORY: 
;       Matthias Liebetanz, Darmstadt University of Technology, Germany, 19 May 1998. 
;- 
;------------------------------------------------------------- 
FUNCTION VIEW_RESULTS, GROUP=Group,$ 
                 SIZE_N=size_n,$ 
                 LON=lon,$ 
                 LAT=lat,$ 
                 DEPTH=depth,$ 
                 GRID_DATE=grid_date,$ 
                 DATA=data,$ 
                 EPOCHE=epoche,$ 
                 GRID_TIMEWIN=grid_timewin,$ 
                 GRID_SHIFT=grid_shift,$ 
                 MAINTITLE=maintitle, $ 
                 PLOTSHOCK=plotshock,$ 
                 SHOCK_TO_PLOT=shock_to_plot,$ 
                 SUBTITLE=subtitle, $ 
                 FILE=file,$ 
                 PATH=path,$ 
                 COLOR=color,$ 
                 ANIMATION=animation,$ 
                 WRITE_GIF=write_gif,$ 
                 NAME=name 
 
 
  saved_P = !P 
 
  !P.CHARSIZE = 1 
  !P.NOCLIP = 1 
 
  epoche_n=N_ELEMENTS(epoche) 
 
  max_val=MAX(data,MIN=min_val) 
 
  IF KEYWORD_SET(PLOTSHOCK) THEN BEGIN 
     X = [-1, -2, 0, 2, 1,2,0,-2] 
     Y = [0, -2, -1, -2, 0,2,1,2] 
     USERSYM,X,Y,/FILL 
  ENDIF 
 
  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0 
 
  junk   = { CW_PDMENU_S, flags:0, name:'' } 
 
  IF (size_n(2) GT 1) THEN BEGIN 
 
     result = XSELECTZ(depth) 
 
     IF N_ELEMENTS(result) EQ 1 THEN BEGIN 
        !P = saved_P 
        RETURN, 0 
     ENDIF 
 
     depth = result(0) 
     z = result(1) 
 
  ENDIF ELSE z=0 
 
 
  IF N_ELEMENTS(data(*,*,0,*)) EQ epoche_n THEN BEGIN 
 
     xtitle_old=!x.title 
     ytitle_old=!y.title 
 
     !x.title = 'Time' 
     !y.title = name 
 
     WINDOW 
 
     JSPLOT, epoche, $ 
             data, $ 
             OFF=off, $ 
             YSTYLE=1 
 
     IF KEYWORD_SET(PLOTSHOCK) THEN BEGIN 
 
        timediff=ABS(shock_to_plot.js-epoche) 
 
        min_timediff = MIN(timediff) 
 
        ishock = WHERE ( timediff EQ min_timediff ) 
 
        OPLOT,[shock_to_plot.js-off,shock_to_plot.js-off],$ 
              [data(ishock),data(ishock)],$ 
              PSYM=8,$ 
              syms=1.5 
 
        PLOTS,shock_to_plot.js-off,data(ishock),$ 
              LINESTYLE=1 
 
        PLOTS,shock_to_plot.js-off,min_val,$ 
              LINESTYLE=1,$ 
              /CONT 
 
     ENDIF 
 
     im=TVRD() 
 
     im_size=[N_ELEMENTS(im(*,0)),N_ELEMENTS(im(0,*))] 
 
     center_x= im_size(0)/2 
 
     XYOUTS,center_x,im_size(1)-15,Maintitle,$ 
                                   ALIGNMENT=0.5, $ 
                                   CHARSIZE=1.4, $ 
                                   /DEVICE 
 
     XYOUTS,center_x,im_size(1)-48,Subtitle,$ 
                                   ALIGNMENT=0.5, $ 
                                   CHARSIZE=1.0, $ 
                                   /DEVICE 
 
     IF KEYWORD_SET(WRITE_GIF) THEN WRITE_GIF, path+'pic.gif',TVRD() 
 
     !x.title = xtitle_old 
     !y.title = ytitle_old 
 
 
     !P = saved_P 
 
     RETURN, 0 
 
  ENDIF 
 
  IF KEYWORD_SET(COLOR) THEN LOADCT,color ELSE LOADCT,27 
 
 
  !X.TITLE='Longitude' 
  !Y.TITLE='Latitude' 
  !Z.TITLE=name 
  !Y.CHARSIZE=1.5 
  !Z.CHARSIZE=1.5 
 
  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $ 
      COLUMN=1, $ 
      MAP=1, $ 
      UVALUE='MAIN13') 
 
  DRAW2 = WIDGET_DRAW( MAIN13, $ 
      RETAIN=0, $ 
      UVALUE='DRAW_2DVIEW', $ 
      XSIZE=350, $ 
      YSIZE=350) 
 
  BUTTON_CANCEL = WIDGET_BUTTON( MAIN13, $ 
      UVALUE='BUTTON_CANCEL', $ 
      VALUE='Cancel') 
 
  WIDGET_CONTROL, MAIN13, /REALIZE 
 
  from_date=dt_tm_fromjs(grid_date(0),FORMAT='0d$.n$.Y$ h$:m$:s$') 
  to_date=dt_tm_fromjs(grid_date(1),FORMAT='0d$.n$.Y$ h$:m$:s$') 
 
 !Z.RANGE=[DOUBLE(min_val),DOUBLE(max_val)] 
 
 
  im=TVRD() 
  im_size=[N_ELEMENTS(im(*,0)),N_ELEMENTS(im(0,*))] 
 
  IF KEYWORD_SET(ANIMATION) THEN BEGIN 
     im=BYTARR(im_size(0),im_size(1),epoche_n) 
  ENDIF 
 
  center_x= im_size(0)/2 
 
  min_lon=MIN(lon,MAX=max_lon) 
  min_lat=MIN(lat,MAX=max_lat) 
 
  ; Force axis-range 
  max_ticks=3 
  IF (size_n(1) MOD 2) EQ 0 THEN add=0 ELSE add=1 
  xtickname=indgen(FIX((size_n(1)+add+1)/FLOAT(max_ticks)))*max_ticks 
  IF N_ELEMENTS(xtickname) GT max_ticks THEN xtickname=xtickname(0:max_ticks-1) 
  xtickname=SCL(min_lon, max_lon, xtickname) 
  min_xtick=MIN(xtickname,MAX=max_xtick) 
  xtickname=STRING(xtickname,FORMAT='(F5.2)') 
  IF (size_n(0) MOD 2) EQ 0 THEN add=0 ELSE add=1 
  ytickname=indgen(FIX((size_n(0)+add+1)/FLOAT(max_ticks)))*max_ticks 
  IF N_ELEMENTS(ytickname) GT max_ticks THEN ytickname=ytickname(0:max_ticks-1) 
  ytickname=SCL(min_lat, max_lat, ytickname) 
  min_ytick=MIN(ytickname,MAX=max_ytick) 
  ytickname=STRING(ytickname,FORMAT='(F5.2)') 
 
 
 
  IF KEYWORD_SET(PLOTSHOCK) THEN BEGIN 
 
     londiff = ABS ( shock_to_plot.lon - lon ) 
 
     min_londiff = MIN ( londiff ) 
 
     ilon = WHERE ( londiff EQ min_londiff ) 
 
 
 
     latdiff=ABS(shock_to_plot.lat-lat) 
 
     min_latdiff = MIN(latdiff) 
 
     ilat = WHERE ( latdiff EQ min_latdiff ) 
 
 
     zdiff=ABS(shock_to_plot.z-depth) 
 
     min_zdiff = MIN(zdiff) 
 
     iz = WHERE ( zdiff EQ min_zdiff ) 
 
 
     timediff=ABS(shock_to_plot.js-epoche) 
 
     min_timediff = MIN(timediff) 
 
     itime = WHERE ( timediff EQ min_timediff ) 
 
     zval = data ( ilon, ilat, iz, itime ) 
 
     norm_data = scl ( 0., 1., data ) ; normalize data (max =1, min=0) 
     norm_zval = norm_data ( ilon, ilat, iz, itime ) 
 
 ENDIF 
 
 
 
  FOR t=0,epoche_n-1 DO BEGIN 
 
      Event=WIDGET_EVENT(BUTTON_CANCEL,BAD_ID=bad,/NOWAIT) 
      IF (event.Id EQ BUTTON_CANCEL) THEN BEGIN 
             answer=XDECISION('Cancel View ?',TITLE='Verify') 
             IF STRLOWCASE(answer) EQ 'y' THEN BEGIN 
                WIDGET_CONTROL,MAIN13,/DESTROY 
                !P = saved_P 
                RETURN,0 
             END 
      ENDIF 
 
 
      SHADE_SURF,data(*,*,z,t),$ 
                 CLIP=[0,0,im_size(0),im_size(1)-100],$ 
                 XTICKNAME=xtickname,$ 
                 XTICKS=N_ELEMENTS(xtickname)-1,$ 
                 XCHARSIZE=1.7,$ 
                 XSTYLE=1,$ 
                 YSTYLE=1,$ 
                 ZSTYLE=1,$ 
                 YTICKNAME=ytickname,$ 
                 YTICKS=N_ELEMENTS(ytickname)-1,$ 
                 YCHARSIZE=1.7,$ 
                 /SAVE 
 
 
      IF KEYWORD_SET(PLOTSHOCK) THEN BEGIN 
 
         PLOT,[min_lon,max_lon],$ 
              [min_lat,max_lat],$ 
              /T3D,$ 
              ZVAL=norm_zval(0),$ 
              /NOERASE,$ 
              XSTYLE=5,$ 
              YSTYLE=5,$ 
              CHARSIZE=1,$ 
              /nodata 
 
         OPLOT,[shock_to_plot.lon,shock_to_plot.lon],$ 
               [shock_to_plot.lat,shock_to_plot.lat],$ 
               /T3D,$ 
               PSYM=8,$ 
               syms=3 
 
         PLOTS,shock_to_plot.lon,shock_to_plot.lat,0.,$ 
               /T3D,$ 
               LINESTYLE=1 
 
         PLOTS,shock_to_plot.lon,shock_to_plot.lat,zval(0),$ 
               /T3D,$ 
               LINESTYLE=1,$ 
               /CONT 
 
 
      ENDIF 
 
      dt_tm_brk,dt_tm_fromjs(epoche(t),FORMAT='0d$.n$.Y$ h$:m$:s$'),date,time 
 
      XYOUTS,center_x,im_size(1)-13,Maintitle,$ 
                                   ALIGNMENT=0.5, $ 
                                   CHARSIZE=1.4, $ 
                                   /DEVICE 
 
      XYOUTS,center_x,im_size(1)-33,Subtitle,$ 
                                   ALIGNMENT=0.5, $ 
                                   CHARSIZE=1.0, $ 
                                   /DEVICE 
 
      XYOUTS,center_x+27,im_size(1)-55,'Date: ' + date + ' Time: ' + time + ' Depth: ' + STRTRIM(STRING(FIX(depth/1000)),1) + ' km' , $ 
                                   ALIGNMENT=0.5, $ 
                                   CHARSIZE=1.0, $ 
                                   /DEVICE 
 
      IF KEYWORD_SET(ANIMATION) THEN BEGIN 
         im(*,*,t)=TVRD() 
      ENDIF 
 
      IF KEYWORD_SET(WRITE_GIF) THEN WRITE_GIF, 'PIC'+STRTRIM(STRING(t+1),1)+'.gif',TVRD() 
 
   ENDFOR 
 
   WIDGET_CONTROL,MAIN13,/DESTROY 
 
   !P = saved_P 
 
  IF KEYWORD_SET(ANIMATION) THEN RETURN,TEMPORARY(im) ELSE RETURN,1 
 
 
END 
