;-------------------------------------------------------------
;+
; NAME:
;       XDECISION
; PURPOSE:
;       Pops up a window with a yes/no Question. The caller-programm is blocked,
;       until the yes or the no button is pressed.
; CATEGORY:
;       Widgets
; CALLING SEQUENCE:
;       answer=XDECISION([question] [,TITLE=title])
; INPUTS:
;       question = Text to be displayed
; KEYWORD PARAMETERS:
;       Keywords:
;       TITLE = If this keyword is set, the title of the window is the string of 'title'.
; OUTPUTS:
;       Returns string either 'N' for No or 'Y' for Yes.
; COMMON BLOCKS:
; NOTES:
; MODIFICATION HISTORY:
;       Matthias Liebetanz, 15. Apr, 1997
;-
;-------------------------------------------------------------
FUNCTION XDECISION, text, TITLE=title

      IF NOT KEYWORD_SET(TITLE) THEN title='Verify'


      yesno=WIDGET_BASE(TITLE=title, $
                        /COLUMN, $
                        MAP=1)

      FOR i=0,N_ELEMENTS(text)-1 DO BEGIN
          question = WIDGET_LABEL( yesno,$
                                   VALUE=text(i))
      ENDFOR

      CASE !VERSION.OS OF

           'windows':   child=WIDGET_BASE(yesno,$  ;IDL 3.6 for windows
                        /ROW)

           ELSE: child=WIDGET_BASE(yesno,$
                        /ROW,$
                        /ALIGN_CENTER)
      ENDCASE

      yes = WIDGET_BUTTON( child, $
          UVALUE='Y', $
          VALUE='Yes')

      no = WIDGET_BUTTON( child, $
          UVALUE='N', $
          VALUE='No')

      answer = ''                                           ; Construct a state variable for this instance.

      WIDGET_CONTROL, WIDGET_INFO(yesno, /CHILD), $
                        SET_UVALUE=answer, /NO_COPY         ; Save the state in the first child's user value.

      WIDGET_CONTROL,yesno,/REALIZE
      WIDGET_CONTROL,yes,/INPUT_FOCUS

      pressed=WIDGET_EVENT(yesno)                           ; Look for arrived events and process them (f.e.YES-Button)

      WIDGET_CONTROL,pressed.id,GET_UVALUE=answer

      WIDGET_CONTROL, yesno,  /DESTROY                      ; Destroy application

      RETURN,answer
END




