;-------------------------------------------------------------
;+
; NAME:
;       XQUAKE
; PURPOSE:
;       Widget-based user-interface for visualization of earthquakes
; CATEGORY:
;       user-specific, mapping
; CALLING SEQUENCE:
;       XQUAKE,GROUP=Group,qu_lat,qu_lon[,TITLE=title][,HIRES=HIRES][,GRID=GRID][,LABEL=LABEL][,XYSIZE=xysize]
; INPUTS:
;       QU_LON = Vektor of longitudes of earthquakes. Value given in degree.degree
;       QU_LAT = Vektor of latitudes of earthquakes. Value given in degree.degree
; KEYWORD PARAMETERS:
;       LIMITS = Vewktor of user-defined limits=[latmin,lonmin,latmax,lonmax]
;                If LIMITS=limits is set, but limits is undefined, the borders of
;                the earthquakes are returned
; MODIFICATION HISTORY:
;       Matthias Liebetanz, TH-Darmstadt, Germany, 27 April, 1998.
;-
;-------------------------------------------------------------

; DO NOT REMOVE THIS COMMENT: BEGIN XQUAKE

PRO XQUAKE_Event, Event

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  CASE Ev OF

  'DRAW': BEGIN
      END
  'Zoom in': BEGIN
      child = WIDGET_INFO( event.handler, /CHILD )          ; Get child-ID
      WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY      ; Get state as user-value of child
      IF NOT DEFINED(state,/STRUCTURE,TAG='newlimits') THEN BEGIN ; Save original borders :
         state=CREATE_STRUCT(state,'newlimits',state.limits)
      ENDIF
      boxarea,x1,y2,x2,y1,FLAG=flag                         ; Call of procedure boxarea (jhu), order: coordinates of left upper point (x1,y2) and right lower point (x2,y1)
      IF (flag EQ 1) OR (x1 EQ x2) THEN GOTO,end_zoom_in
      dev=[0,0,state.xysize(0),state.xysize(1)]         ; in xysize the window size in pixel is stored (device coordinates)
      box=[x1,x2,y1,y2]                                     ; order coordinates of left lower point (x1,y1) and right upper point (x2,y2)
      state.newlimits=IPOLBOX(dev,state.newlimits,box)      ; Get the new data-coordinates by linear interpolatint
      WSET,state.draw
      MAP_QUAKE,state.qu_lat,$                              ; Draw map zoomed-in
                state.qu_lon,$
                LIMITS=state.newlimits,$
                HIRES=state.hires,$
                GRID=state.grid,$
                LABEL=state.label
end_zoom_in:
      WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Save state before exit
      END
  'Zoom out': BEGIN
      child = WIDGET_INFO( event.handler, /CHILD )          ; Get child-ID
      WIDGET_CONTROL, child, GET_UVALUE=state,/NO_COPY      ; Get state as user-value of child
      IF NOT DEFINED(state,/STRUCTURE,TAG='newlimits') THEN BEGIN    ; New borders :
         state=CREATE_STRUCT(state,'newlimits',[state.limits(0)-6.,state.limits(1)+6.,$
                                                state.limits(2)-3.,state.limits(3)+3.])
      ENDIF ELSE BEGIN
         state.newlimits(0)=state.newlimits(0)-20
         state.newlimits(1)=state.newlimits(1)+20.
         state.newlimits(2)=state.newlimits(2)-10.
         state.newlimits(3)=state.newlimits(3)+10.
      ENDELSE
      WSET,state.draw
      MAP_QUAKE,state.qu_lat,$                              ; Draw map zoomed-out
                state.qu_lon,$
                LIMITS=state.newlimits,$
                HIRES=state.hires,$
                GRID=state.grid,$
                LABEL=state.label
      WIDGET_CONTROL, child, SET_UVALUE=state,/NO_COPY      ; Save state before exit
      END
  'Exit': BEGIN
      Print, 'Event for Exit'
      WIDGET_CONTROL,event.top,/DEST
      END
  ; Event for Epoche
  'Epoche': PDMENU26_Event, Event
  ENDCASE
END


; DO NOT REMOVE THIS COMMENT: END XQUAKE
; CODE MODIFICATIONS MADE BELOW THIS COMMENT WILL BE LOST.




PRO XQUAKE,GROUP=Group,XQBASE=XQBASE,qu_lat,qu_lon,TITLE=title,HIRES=HIRES,GRID=GRID,LABEL=LABEL,XYSIZE=xysize

  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

  junk   = { CW_PDMENU_S, flags:0, name:'' }

  IF NOT KEYWORD_SET(TITLE) THEN title='XQuake'
  IF NOT KEYWORD_SET(HIRES) THEN HIRES=0
  IF NOT KEYWORD_SET(GRID) THEN GRID=0
  IF NOT KEYWORD_SET(LABEL) THEN LABEL=0
  IF NOT KEYWORD_SET(XYSIZE) THEN xysize=[590,450]


  XQBASE = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, $
      UVALUE='XQBASE',$
      TITLE=title)

  DRAW = WIDGET_DRAW( XQBASE, $
      BUTTON_EVENTS=0, $
      MOTION_EVENTS=0, $
      RETAIN=2, $
      UVALUE='DRAW', $
      XSIZE=xysize(0), $
      YSIZE=xysize(1))

  BASE = WIDGET_BASE(XQBASE, $
      ROW=1, $
      MAP=1, $
      TITLE='SliderBase', $
      UVALUE='BASE')

  ZOOM_IN = WIDGET_BUTTON( BASE, $
      UVALUE='Zoom in', $
      VALUE='zoom in')

  ZOOM_OUT = WIDGET_BUTTON( BASE, $
      UVALUE='Zoom out', $
      VALUE='zoom out')

  EXITB = WIDGET_BUTTON( BASE, $
      UVALUE='Exit', $
      VALUE='Exit')

  WIDGET_CONTROL, XQBASE, /REALIZE

  ; Get drawable window index

  COMMON DRAW_Comm, DRAW_Id
  WIDGET_CONTROL, DRAW, GET_VALUE=DRAW_Id


  MAP_QUAKE,qu_lat,$            ; First call. limits is initialized.
            qu_lon,$
            LIMITS=limits,$
            HIRES=hires,$
            GRID=grid,$
            LABEL=label


  state=CREATE_STRUCT('draw',DRAW_id,$
                      'limits',limits,$
                      'qu_lon',qu_lon,$
                      'qu_lat',qu_lat,$
                      'HIRES',HIRES,$
                      'GRID',GRID,$
                      'LABEL',LABEL,$
                      'xysize',xysize)

  child=WIDGET_INFO(XQBASE,/CHILD)
  WIDGET_CONTROL,child,SET_UVALUE=state
  XMANAGER,'XQBASE',XQBASE,EVENT_HANDLER='XQUAKE_Event'

END



