/*
olap.c overlaps cubes in space and time

dependencies:
- placewindow.c
- timewindow.c
- deg2m.c
- matchin.c
- dsqu.c
*/
#include <windows.h>
#include <math.h>

// declaring prototypes
int place_window( double grid, double a, double quake, double b);
int timewindow(double quake_js,double epoche,double grid_timewin);
double deg2m( double deg);
double matchin( double grid, double a, double quake, double b);
double dsqu( double argument);
/*************************************************************/
long FAR PASCAL _export olap(long lArgc, LPVOID lpvArgv)
{
	/* input variables */
	double		far * grid_lat;
	unsigned long			grid_lat_n;
	double		far * grid_lon;
	unsigned long			grid_lon_n;
	double		far * grid_z;
	unsigned long  		grid_z_n;
	double		grid_length;
	double		far * grid_date;
	double		 epoche;
	unsigned long			epoches_n;
	double		grid_timewin;
	double		grid_shift;
	double		far * quake_lat;
	unsigned long			quake_n;
	double		far * quake_lon;
	double		far * quake_z;
	double		quake_length;
	unsigned long   		far * quake_quality;
	double  		far * quake_js;
	long	  		switch_normalize;
	long	  		switch_quiescence;
	long	  		switch_err;
	double      far * sigxy;
	double      far * sigz;
	double      far * sigt;
	double		far * s;
	double		s_n;
	double		far * err_s;
	double		far * quiescence;
	double		far * err_quiescence;
	long			far * degree_freedom;

	/* local variables */
	long	   	far * far * lplplArgv;
	unsigned long grid_elements_n;
	double small_length;
	unsigned long i;
	unsigned long j;
	unsigned long k;
	unsigned long q;
	unsigned long t;
	double lx;
	double ly;
	double lz;
	double lt;
	double dt;

	unsigned long hallo=0;

	double cubus=1.;


	/* change lpVoid to lpLong */
	lplplArgv = (long far * far *)lpvArgv;
	grid_lat = lplplArgv[0];
	grid_lat_n = * lplplArgv[1];
	grid_lon = lplplArgv[2];
	grid_lon_n = * lplplArgv[3];
	grid_z = lplplArgv[4];
	grid_z_n = * lplplArgv[5];
	grid_length = * (double far *) (lplplArgv)[6];
	grid_date = lplplArgv[8];
	epoche = * (double far *) lplplArgv[10];
	epoches_n = * lplplArgv[11];
	grid_timewin = * (double far *) lplplArgv[12];
	grid_shift = * (double far *) lplplArgv[14];
	quake_lat = lplplArgv[16];
	quake_n = * lplplArgv[17];
	quake_lon = lplplArgv[18];
	quake_z = lplplArgv[20];
	quake_length = * (double far *) lplplArgv[22];
	quake_quality = lplplArgv[24];
	quake_js = lplplArgv[26];
	switch_normalize = * lplplArgv[28];
	switch_quiescence = * lplplArgv[30];
	switch_err = * lplplArgv[32];
	sigxy = lplplArgv[34];
	sigz = lplplArgv[36];
	sigt = lplplArgv[38];
	s = lplplArgv[40];
	s_n = * lplplArgv[41];
	err_s = lplplArgv[42];
	quiescence = lplplArgv[44];
	err_quiescence = lplplArgv[46];
 	degree_freedom = lplplArgv[48];

	grid_elements_n = grid_lat_n * grid_lon_n * grid_z_n;

	for (i=0;i<s_n;i++)
	{
		s[i] = 0.;
	}
	if (switch_err)
	{
		for (i=0;i<s_n;i++)
		{
			err_s[i] = 0.;
         degree_freedom[i] = 0;
		}
	}


		for (q=0;q<quake_n;q++)
		{
			/* is earthquake in the influence-area of the current epoche? */
			if (timewindow(quake_js[q],epoche,grid_timewin))
			{
				/* is earthquake in the influence-area of the actual grid-point? */
				for (k=0;k<grid_z_n;k++)
				{
					if (placewindow(grid_z[k],grid_length,quake_z[q],quake_length))
					{
						for (i=0;i<grid_lat_n;i++)
						{
							if(placewindow(deg2m(grid_lat[i]),grid_length,deg2m(quake_lat[q]),quake_length))
							{
								for (j=0;j<grid_lon_n;j++)
								{
									if (placewindow(deg2m(grid_lon[j]),grid_length,deg2m(quake_lon[q]),quake_length))
									{
										lx=matchin(deg2m(grid_lat[i]),grid_length,deg2m(quake_lat[q]),quake_length);
										ly=matchin(deg2m(grid_lon[j]),grid_length,deg2m(quake_lon[q]),quake_length);
										lz=matchin(grid_z[k],grid_length,quake_z[q],quake_length);
										lt=matchin(epoche,grid_timewin,quake_js[q],grid_timewin);

										//This is to compute overlapping
										s[k*grid_lat_n*grid_lon_n+i*grid_lon_n+j] += lx*ly*lz*lt;


										//Now follows errors of overlapping
										if (switch_err)
										{
											err_s[k*grid_lat_n*grid_lon_n+i*grid_lon_n+j] +=
											dsqu(sigxy[quake_quality[q]]) * (dsqu(ly*lz*lt) + dsqu(lx*lz*lt)) +
											dsqu(sigz[quake_quality[q]]) * dsqu(lx*ly*lt) +
											dsqu(sigt[quake_quality[q]]) * dsqu(lx*ly*lz) ;
                                 degree_freedom[k*grid_lat_n*grid_lon_n+i*grid_lon_n+j] += 1;
										}
									}
								}
							}
						}
					}
				}
			}
		}

	if (switch_normalize)
		cubus = grid_length * grid_length * grid_length * grid_timewin;
	else
		cubus = 1.;

	for (i=0;i<s_n;i++)
	{
		s[i] = s[i] / cubus;
	}

	if (switch_quiescence)
	{
		for (i=0;i<s_n;i++)
		{
			if (s[i] > 0)
				quiescence[i] = - 1 / s[i];

			else

				quiescence[i] = 0.;

		}

	}


	if (switch_err)
	{
		for (i=0;i<s_n;i++)

		{
			err_s[i] = sqrt(err_s[i]/dsqu(cubus));

			if (switch_quiescence)

				err_quiescence[i] = quiescence[i] * cubus;

		}
	}


return hallo;
}


