;-------------------------------------------------------------  
;+  
; NAME:  
;       CW_PROGRESS  
; PURPOSE:  
;       Compound-Widget for displaying progress of computing  
; CATEGORY:  
;       Widgets  
; CALLING SEQUENCE:  
;       CW_PROGRESS, PARENT[, VALUE=VALUE] [,TITLE=TITLE]  
; INPUTS:  
;       PARENT = Id of Parent-widget of CW_SHOWTEXT.  
; KEYWORD PARAMETERS:  
;       Keywords:  
;       TITLE = If this keyword is set, the string becomes the title of the widged-window.  
;       VALUE = Text-String to be displayed.  
; OUTPUTS:  
; COMMON BLOCKS:  
; NOTES:  
; MODIFICATION HISTORY:  
;       Matthias Liebetanz, 05. Apr, 1997  
;-  
;-------------------------------------------------------------  
PRO CW_PROGRESS_EVENT,event  
  
     WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev  
  
     CASE Ev OF  
  
      'Progress.Cancel': BEGIN  
      WIDGET_CONTROL, Event.Top,  /DESTROY                ; Destroy application  
  
      END  
      ENDCASE  
  
END  
  
  
PRO CW_PROGRESS_SET_VALUE, id, value                      ; This is the SET_VALUE routine for CW_PROGRESS.  
                                                          ; The number and type of the arguments is defined by WIDGET_CONTROL.  
                                                          ; Id is the widget ID of a CW_PROGRESS, and value is the users requested value.  
  
    stash = WIDGET_INFO(id, /CHILD)                          ; Get the ID of the first child of the CW_DICE widget.  
                                                         ; This is where the state information is stored.  
  
    WIDGET_CONTROL, stash, GET_UVALUE=state, /NO_COPY     ; Get the state structure.  
  
    state.value=value  
  
    WIDGET_CONTROL, stash, SET_VALUE=value               ; Set the new string on the label  
                                                         ; We take advantage of the fact that stash must be the widget ID  
                                                         ; of the label widget  
  
    WIDGET_CONTROL, stash, SET_UVALUE=state, /NO_COPY    ; Restore the state in the child UVALUE.  
  
    END  
  
FUNCTION CW_PROGRESS, group, TITLE=title, VALUE=value  
  
    IF NOT KEYWORD_SET(TITLE) THEN title='Progress made'  
  
    IF NOT KEYWORD_SET(VALUE) THEN value=''  
  
    state={ value:value }                                ; Construct a state variable for this instance.  
  
    progress=WIDGET_BASE(GROUP_LEADER=group, $           ; Create the base widget  
                        TITLE=title, $  
                        PRO_SET_VALUE='CW_PROGRESS_SET_VALUE', $  
                        /COLUMN)  
  
    CASE !VERSION.OS OF      
         'windows': BEGIN 
                    info=WIDGET_LABEL(progress,VALUE=value) 
                    END      
          ELSE:     BEGIN 
                    info=WIDGET_LABEL(progress,VALUE=value,/DYNAMIC_RESIZE)  
                    END 
    ENDCASE 
 
    cancel=WIDGET_BUTTON(progress, $  
                        VALUE='CANCEL', $  
                        UVALUE='Progress.Cancel')  
  
    WIDGET_CONTROL, WIDGET_INFO(progress, /CHILD), $     ; Save the state in the first child's user value.  
                                                         ; Notice the use of the NO_COPY keyword for efficiency.  
    SET_UVALUE=state, /NO_COPY  
  
    WIDGET_CONTROL,progress,/REALIZE  
  
    XMANAGER,'CW_PROGRESS',progress, $  
             EVENT_HANDLER='CW_PROGRESS_EVENT'            ; an event function  
  
    RETURN, progress                                      ; The result of a compound widget is always the ID of its topmost widget.  
  
    END  
  
  
  
  
  
  
  
  
  
